"""
Bulk operations for contact management
This module provides highly optimized implementations for operations on multiple contacts
"""
import typer
from db.database import Session, engine
from models.core import Contact
from sqlalchemy.dialects.sqlite import insert as sqlite_insert
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy import select, text, bindparam, and_
from typing import List, Dict, Any, Optional, Tuple, Set
import uuid
import time
from itertools import islice
from contextlib import contextmanager
import multiprocessing
from concurrent.futures import ThreadPoolExecutor

# Optimal batch size based on benchmarks
OPTIMAL_BATCH_SIZE = 500

# Configure how many threads to use for parallel processing
# Uses CPU count - 1 but minimum of 2, maximum of 8
NUM_WORKERS = min(max(multiprocessing.cpu_count() - 1, 2), 8)

def _chunk_list(items, chunk_size):
    """Split a list into chunks of specified size"""
    iterator = iter(items)
    while True:
        chunk = list(islice(iterator, chunk_size))
        if not chunk:
            break
        yield chunk

def bulk_add_contacts(contacts_data: List[Dict[str, Any]]) -> List[Contact]:
    """
    Add multiple contacts in a single transaction with optimized batching
    
    Args:
        contacts_data: List of dictionaries with contact data
        
    Returns:
        List of created Contact objects with their IDs
    """
    if not contacts_data:
        return []
        
    start_time = time.time()
    contact_objects = []
    
    # Determine optimal batch size based on data size
    batch_size = min(OPTIMAL_BATCH_SIZE, max(100, len(contacts_data) // 10))
    
    # Get a database session
    session = Session()
    try:
        # Process in efficient batches
        for i, batch in enumerate(_chunk_list(contacts_data, batch_size)):
            # Create Contact objects for this batch
            batch_objects = [Contact(**data) for data in batch]
            
            # Add all contacts in this batch at once
            session.add_all(batch_objects)
            
            # Extend our result list
            contact_objects.extend(batch_objects)
            
            # Commit each batch to avoid memory issues with very large datasets
            # and to ensure early batches aren't lost if later ones fail
            session.commit()
            
            # Progress update for large batches
            if len(contacts_data) > 1000 and i % 5 == 0:
                processed = min((i + 1) * batch_size, len(contacts_data))
                typer.echo(f"Progress: {processed}/{len(contacts_data)} contacts processed")
        
        duration = time.time() - start_time
        # Print success message with performance stats
        typer.echo(f"Added {len(contact_objects)} contacts successfully in {duration:.2f} seconds")
        return contact_objects
    except SQLAlchemyError as e:
        session.rollback()
        typer.echo(f"Error adding contacts in bulk: {str(e)}", err=True)
        raise
    finally:
        session.close()

def _process_update_batch(updates: List[Dict[str, Any]], session=None) -> int:
    """
    Process a batch of updates in a separate session
    
    Args:
        updates: List of update dictionaries
        session: Existing session or None to create a new one
        
    Returns:
        Number of contacts updated
    """
    close_session = False
    if session is None:
        session = Session()
        close_session = True
        
    try:
        updated_count = 0
        
        # Extract IDs
        ids = [uuid.UUID(u['local_id']) if isinstance(u['local_id'], str) else u['local_id'] 
              for u in updates]
        
        # Get contacts to update in a single query
        contacts = session.query(Contact).filter(Contact.local_id.in_(ids)).all()
        contact_map = {c.local_id: c for c in contacts}
        
        # Apply updates
        for update in updates:
            contact_id = uuid.UUID(update['local_id']) if isinstance(update['local_id'], str) else update['local_id']
            if contact_id in contact_map:
                contact = contact_map[contact_id]
                for key, value in update.items():
                    if key != 'local_id':
                        setattr(contact, key, value)
                updated_count += 1
                
        # Don't commit here if using an external session
        if close_session:
            session.commit()
            
        return updated_count
    except:
        # Only rollback if we own the session
        if close_session:
            session.rollback()
        raise
    finally:
        # Only close if we own the session
        if close_session:
            session.close()

def bulk_update_contacts(update_data: List[Dict[str, Any]]) -> int:
    """
    Update multiple contacts with optimized batching
    
    Args:
        update_data: List of dictionaries with contact data, must include 'local_id'
        
    Returns:
        Number of contacts updated
    """
    if not update_data:
        return 0
        
    start_time = time.time()
    
    # Get a database session
    session = Session()
    try:
        updated_count = 0
        
        # Group updates by attributes being updated for maximum efficiency
        updates_by_attrs = {}
        for update in update_data:
            if 'local_id' not in update:
                continue
                
            # Create a key based on the attributes being updated (excluding ID)
            attrs = tuple(sorted([k for k in update.keys() if k != 'local_id']))
            if attrs not in updates_by_attrs:
                updates_by_attrs[attrs] = []
            updates_by_attrs[attrs].append(update)
        
        # Determine if we should use parallel processing
        use_parallel = len(update_data) > 1000 and len(updates_by_attrs) > 1
        
        if use_parallel and NUM_WORKERS > 1:
            # Process groups in parallel for very large datasets
            with ThreadPoolExecutor(max_workers=NUM_WORKERS) as executor:
                results = []
                for attrs, updates in updates_by_attrs.items():
                    # Split into optimal batch sizes
                    for batch in _chunk_list(updates, OPTIMAL_BATCH_SIZE):
                        results.append(executor.submit(_process_update_batch, batch))
                
                # Collect results
                for future in results:
                    updated_count += future.result()
        else:
            # Process serially for smaller datasets
            for attrs, updates in updates_by_attrs.items():
                # Process in batches for better performance
                for batch in _chunk_list(updates, OPTIMAL_BATCH_SIZE):
                    updated_count += _process_update_batch(batch, session)
        
        # Final commit
        session.commit()
        
        duration = time.time() - start_time
        # Print performance stats
        typer.echo(f"Updated {updated_count} contacts in {duration:.2f} seconds")
        return updated_count
    except SQLAlchemyError as e:
        session.rollback()
        typer.echo(f"Error updating contacts in bulk: {str(e)}", err=True)
        raise
    finally:
        session.close()

def bulk_delete_contacts(contact_ids: List[str]) -> int:
    """
    Delete multiple contacts using optimized batching
    
    Args:
        contact_ids: List of contact IDs to delete
        
    Returns:
        Number of contacts deleted
    """
    if not contact_ids:
        return 0
        
    start_time = time.time()
    
    # Optimal batch size for deletes
    batch_size = min(OPTIMAL_BATCH_SIZE, max(100, len(contact_ids) // 5))
    
    # Get a database session
    session = Session()
    try:
        total_deleted = 0
        
        # Process in batches to avoid SQLite parameter limits and memory issues
        for batch in _chunk_list(contact_ids, batch_size):
            # Convert string IDs to UUID objects
            uuid_ids = [uuid.UUID(id) if isinstance(id, str) else id for id in batch]
            
            # Execute batch delete
            result = session.query(Contact).filter(Contact.local_id.in_(uuid_ids)).delete(synchronize_session=False)
            total_deleted += result
            
            # Commit each batch
            session.commit()
        
        duration = time.time() - start_time
        # Print success message with performance stats
        typer.echo(f"Deleted {total_deleted} contacts in {duration:.2f} seconds")
        return total_deleted
    except SQLAlchemyError as e:
        session.rollback()
        typer.echo(f"Error deleting contacts in bulk: {str(e)}", err=True)
        raise
    finally:
        session.close()

def _execute_bulk_upsert(batch: List[Dict[str, Any]], key_field: str, session=None) -> Tuple[int, int]:
    """
    Execute optimized bulk upsert on a batch of contacts
    
    Args:
        batch: List of contact data dictionaries
        key_field: Key field for matching existing records
        session: SQLAlchemy session or None to create a new one
        
    Returns:
        Tuple of (inserted_count, updated_count)
    """
    close_session = False
    if session is None:
        session = Session()
        close_session = True
        
    try:
        inserted, updated = 0, 0
        
        # Get all key values for this batch
        key_values = [data.get(key_field) for data in batch if data.get(key_field)]
        
        # Find all existing records in a single query
        existing_records = {}
        if key_values:
            stmt = select(Contact).where(getattr(Contact, key_field).in_(key_values))
            for contact in session.execute(stmt).scalars().all():
                existing_records[getattr(contact, key_field)] = contact
        
        # Process batch
        for data in batch:
            key_value = data.get(key_field)
            if not key_value:
                continue
                
            if key_value in existing_records:
                # Update existing record
                contact = existing_records[key_value]
                for k, v in data.items():
                    if k != 'local_id':  # Don't update the primary key
                        setattr(contact, k, v)
                updated += 1
            else:
                # Insert new record
                contact = Contact(**data)
                session.add(contact)
                inserted += 1
        
        # Commit if we own the session
        if close_session:
            session.commit()
            
        return (inserted, updated)
    except:
        # Only rollback if we own the session
        if close_session:
            session.rollback()
        raise
    finally:
        # Only close if we own the session
        if close_session:
            session.close()

def upsert_contacts(contacts_data: List[Dict[str, Any]], key_field: str = 'email') -> Dict[str, int]:
    """
    Insert or update contacts with optimized batching and parallel processing
    
    Args:
        contacts_data: List of dictionaries with contact data
        key_field: Field to use as the unique key (default: email)
        
    Returns:
        Dictionary with counts of inserted and updated records
    """
    if not contacts_data:
        return {'inserted': 0, 'updated': 0}
    
    start_time = time.time()
    
    # Get a database session
    session = Session()
    try:
        result = {'inserted': 0, 'updated': 0}
        
        # Determine batch size based on data size
        batch_size = min(OPTIMAL_BATCH_SIZE, max(100, len(contacts_data) // 10))
        
        # Determine if we should use parallel processing
        use_parallel = len(contacts_data) > 2000 and NUM_WORKERS > 1
        
        if use_parallel:
            # Process batches in parallel for very large datasets
            with ThreadPoolExecutor(max_workers=NUM_WORKERS) as executor:
                futures = []
                for batch in _chunk_list(contacts_data, batch_size):
                    futures.append(executor.submit(_execute_bulk_upsert, batch, key_field))
                
                # Collect results
                for future in futures:
                    inserted, updated = future.result()
                    result['inserted'] += inserted
                    result['updated'] += updated
        else:
            # Process serially for smaller datasets
            for batch in _chunk_list(contacts_data, batch_size):
                inserted, updated = _execute_bulk_upsert(batch, key_field, session)
                result['inserted'] += inserted
                result['updated'] += updated
        
        # Commit final changes
        session.commit()
        
        duration = time.time() - start_time
        # Print performance stats
        typer.echo(f"Upserted {result['inserted'] + result['updated']} contacts "
                  f"({result['inserted']} inserted, {result['updated']} updated) "
                  f"in {duration:.2f} seconds")
        
        return result
    except SQLAlchemyError as e:
        session.rollback()
        typer.echo(f"Error upserting contacts: {str(e)}", err=True)
        raise
    finally:
        session.close()