import uuid
import typer
from db.database import Session
from models import Contact
from sqlalchemy import exc

def delete_contact(contact_id: str):
    """
    Delete a specific contact by ID
    """
    # Get a database session
    session = Session()
    try:
        # Convert string to UUID
        contact_uuid = uuid.UUID(contact_id)
        
        # Query the contact by ID
        contact = session.query(Contact).filter(Contact.local_id == contact_uuid).first()
        
        if not contact:
            typer.echo(f"Contact with ID {contact_id} not found.")
            return False
        
        # Display warning and ask for confirmation
        typer.echo(f"WARNING: You are about to delete contact: {contact.full_name} ({contact.email})")
        typer.echo("This action will CASCADE to all backups when sync happens.")
        typer.echo("To confirm deletion, type 'DELETE' (all uppercase):")
        
        confirmation = typer.prompt("")
        
        if confirmation != "DELETE":
            typer.echo("Deletion canceled.")
            return False
        
        # Delete the contact
        session.delete(contact)
        session.commit()
        typer.echo(f"Contact '{contact.full_name}' has been deleted.")
        return True
    
    except ValueError:
        typer.echo(f"Invalid UUID format: {contact_id}", err=True)
        return False
    except exc.SQLAlchemyError as e:
        session.rollback()
        typer.echo(f"Database error deleting contact: {str(e)}", err=True)
        return False
    except Exception as e:
        typer.echo(f"Error deleting contact: {str(e)}", err=True)
        raise
    finally:
        session.close()

def delete_all_contacts():
    """
    Delete ALL contacts from the database
    """
    # Get a database session
    session = Session()
    try:
        # Count total contacts
        contact_count = session.query(Contact).count()
        
        if contact_count == 0:
            typer.echo("No contacts found in the database.")
            return False
        
        # Display warning and ask for confirmation
        typer.echo(f"WARNING: You are about to delete ALL {contact_count} contacts from the database.")
        typer.echo("This action will CASCADE to all backups when sync happens.")
        typer.echo("This action cannot be undone.")
        typer.echo("To confirm deletion of ALL contacts, type 'DELETE' (all uppercase):")
        
        confirmation = typer.prompt("")
        
        if confirmation != "DELETE":
            typer.echo("Deletion canceled.")
            return False
        
        # Delete all contacts
        deleted_count = session.query(Contact).delete()
        session.commit()
        typer.echo(f"All {deleted_count} contacts have been deleted.")
        return True
    
    except exc.SQLAlchemyError as e:
        session.rollback()
        typer.echo(f"Database error deleting contacts: {str(e)}", err=True)
        return False
    except Exception as e:
        typer.echo(f"Error deleting contacts: {str(e)}", err=True)
        raise
    finally:
        session.close()