"""
Utility to fix contact names that weren't properly imported.
This module helps repair contacts with missing names by attempting to
extract them from sync_notes or resyncing with macOS Contacts.
"""

import typer
import platform
from sqlalchemy import or_
from db.database import Session
from models.core import Contact

def find_contacts_with_missing_names():
    """
    Find all contacts that don't have a name.
    
    Returns:
        List of Contact objects without names, sorted by creation date.
    """
    session = Session()
    try:
        # Query contacts with null or empty first AND last name
        contacts = session.query(Contact).filter(
            or_(
                Contact.name_first.is_(None),
                Contact.name_first == ""
            ),
            or_(
                Contact.name_last.is_(None),
                Contact.name_last == ""
            )
        ).order_by(Contact.created_at.desc()).all()
        
        return contacts
    finally:
        session.close()

def list_contacts_with_missing_names():
    """List all contacts without names."""
    contacts = find_contacts_with_missing_names()
    
    if not contacts:
        typer.echo("All contacts have names. Nothing to fix! 🎉")
        return []
    
    # Display a formatted table of contacts without names
    typer.echo(f"Found {len(contacts)} contacts without names:\n")
    
    # Print header
    typer.echo(f"{'ID':<36} | {'Email':<25} | {'Created At':<19} | {'Sync Notes':<20}")
    typer.echo("-" * 105)
    
    # Print each contact with available information
    for contact in contacts:
        created_at = contact.created_at.strftime("%Y-%m-%d %H:%M") if contact.created_at else "N/A"
        sync_notes = (contact.sync_notes[:20] + "...") if contact.sync_notes and len(contact.sync_notes) > 20 else (contact.sync_notes or "N/A")
        email = (contact.email[:25] + "...") if contact.email and len(contact.email) > 25 else (contact.email or "N/A")
        
        typer.echo(f"{str(contact.local_id):<36} | "
                  f"{email:<25} | "
                  f"{created_at:<19} | "
                  f"{sync_notes:<20}")
    
    return contacts

def fix_missing_names():
    """
    Fix contacts with missing names.
    
    This function attempts to fix missing names by:
    1. Checking if we're on macOS and have access to Contacts
    2. For each contact with missing names, check if it has a sync_notes with apple_id
    3. If found, try to fetch the contact from macOS Contacts and update the name
    """
    # Check if we're on macOS
    if platform.system() != 'Darwin':
        typer.echo("This function requires macOS to access the Contacts app.")
        typer.echo("Since you're not on macOS, we can't automatically fix contact names.")
        return
    
    try:
        # Try to import Contacts
        import Contacts
    except ImportError:
        typer.echo("The pyobjc-framework-Contacts package is required but not installed.")
        typer.echo("Please install it with: pip install pyobjc-framework-Contacts")
        return
    
    # Get contacts with missing names
    contacts = find_contacts_with_missing_names()
    
    if not contacts:
        typer.echo("All contacts have names. Nothing to fix! 🎉")
        return
    
    typer.echo(f"Found {len(contacts)} contacts with missing names. Attempting to fix...")
    
    # Get the default ContactsBook
    contacts_book = Contacts.CNContactStore.alloc().init()
    
    # Request authorization to access contacts
    # First check the current authorization status (class method)
    status = Contacts.CNContactStore.authorizationStatusForEntityType_(Contacts.CNEntityTypeContacts)
    
    # If not authorized, explain and exit
    if status != Contacts.CNAuthorizationStatusAuthorized:
        typer.echo("Permission to access Contacts is required but not granted.")
        typer.echo("Please run 'contactcmd sync mac' first to set up permissions.")
        return
    
    # Define which contact properties to fetch
    keys_to_fetch = [
        Contacts.CNContactGivenNameKey,
        Contacts.CNContactFamilyNameKey,
        Contacts.CNContactIdentifierKey,
    ]
    
    # Process each contact
    fixed_count = 0
    
    session = Session()
    try:
        for contact in contacts:
            if not contact.sync_notes or 'apple_id:' not in contact.sync_notes:
                typer.echo(f"Contact {contact.local_id} has no Apple ID in sync_notes. Can't fix.")
                continue
            
            # Extract Apple ID from sync_notes
            try:
                apple_id_section = [s for s in contact.sync_notes.split(';') if 'apple_id:' in s]
                if not apple_id_section:
                    continue
                    
                apple_id = apple_id_section[0].split('apple_id:')[1].strip()
                
                # Create predicate to fetch by ID
                predicate = Contacts.CNContact.predicateForContactsWithIdentifiers_([apple_id])
                
                # Try to fetch the contact from macOS Contacts
                mac_contacts = contacts_book.unifiedContactsMatchingPredicate_keysToFetch_error_(
                    predicate, keys_to_fetch, None
                )
                
                if mac_contacts and len(mac_contacts) > 0:
                    mac_contact = mac_contacts[0]
                    
                    # Extract the name
                    first_name = mac_contact.givenName()
                    last_name = mac_contact.familyName()
                    
                    # Update the contact
                    contact.name_first = first_name
                    contact.name_last = last_name
                    contact.is_dirty = True
                    fixed_count += 1
                    
                    typer.echo(f"Fixed contact: {contact.local_id} → {first_name} {last_name}")
                else:
                    typer.echo(f"Contact with Apple ID {apple_id} not found in macOS Contacts.")
            except Exception as e:
                typer.echo(f"Error processing contact {contact.local_id}: {str(e)}")
        
        # Commit all changes
        session.commit()
        typer.echo(f"\nFixed {fixed_count} out of {len(contacts)} contacts with missing names.")
        
        if fixed_count < len(contacts):
            typer.echo(f"Could not fix {len(contacts) - fixed_count} contacts. They may need manual editing.")
    finally:
        session.close()

if __name__ == "__main__":
    # Can be run directly for testing
    fix_missing_names()