import typer
from db.database import Session
from models import Contact
from sqlalchemy import asc, desc, select, bindparam, func
from enum import Enum
from typing import Optional, Dict, List, Tuple
from functools import lru_cache
from db.cache import cached_query, smart_cached_query
import math

class SortField(str, Enum):
    FIRST_NAME = "first_name"
    LAST_NAME = "last_name"
    EMAIL = "email"
    CREATED = "created"

class SortOrder(str, Enum):
    ASC = "asc"
    DESC = "desc"

# Default page size
DEFAULT_PAGE_SIZE = 20

# Map the sort fields to the model attributes for prepared statements
SORT_FIELD_MAP = {
    "first_name": Contact.name_first,
    "last_name": Contact.name_last,
    "email": Contact.email,
    "created": Contact.created_at
}

# Create a dictionary of prepared statements for each sort field and order combination
# These statements are used for non-paginated queries
prepared_statements: Dict = {}

# Generate prepared statements for sorting in ascending order
for field_name, field in SORT_FIELD_MAP.items():
    # Ascending order
    prepared_statements[f"{field_name}_asc"] = select(Contact).order_by(asc(field))
    # Descending order
    prepared_statements[f"{field_name}_desc"] = select(Contact).order_by(desc(field))

# Default statement with no sorting
default_statement = select(Contact)

# Function to generate pagination statements dynamically to avoid caching too many statements
def get_paginated_statement(sort_by: Optional[str], order: Optional[str], 
                           page: int, page_size: int) -> Tuple[object, int, int]:
    """
    Get a paginated statement based on sort settings
    
    Args:
        sort_by: Field to sort by
        order: Sort order
        page: Page number (1-based)
        page_size: Number of items per page
        
    Returns:
        Tuple of (statement, offset, limit)
    """
    # Calculate offset and limit
    offset = (page - 1) * page_size
    limit = page_size
    
    # Determine which field to sort by
    sort_field = None
    if sort_by and sort_by in SORT_FIELD_MAP:
        sort_field = SORT_FIELD_MAP[sort_by]
    else:
        # Default to ID if sort_by is invalid or not specified
        sort_field = Contact.local_id
    
    # Determine sort order
    normalized_order = order.lower() if order else "asc"
    if normalized_order not in ("asc", "desc"):
        normalized_order = "asc"
    
    # Create the statement with pagination
    if normalized_order == "asc":
        stmt = select(Contact).order_by(asc(sort_field)).offset(offset).limit(limit)
    else:
        stmt = select(Contact).order_by(desc(sort_field)).offset(offset).limit(limit)
    
    return (stmt, offset, limit)

@cached_query(cache_type="reference", ttl=300)
def get_contact_count():
    """Get count of contacts (cached with longer TTL for reference data)"""
    with Session() as session:
        return session.query(Contact).count()

@cached_query(cache_type="standard", ttl=120)
def _list_contacts_cached(sort_by: Optional[str] = None, order: Optional[str] = "asc"):
    """
    Cached implementation of contact listing with optional sorting
    
    Uses granular caching strategy with context-aware TTL settings.
    
    Args:
        sort_by: Field to sort by
        order: Sort order
        
    Returns:
        List of all contacts with sorting applied
    """
    session = Session()
    try:
        # Determine which prepared statement to use
        stmt_key = None
        if sort_by and sort_by in SORT_FIELD_MAP:
            # Normalize order
            normalized_order = order.lower() if order else "asc"
            if normalized_order not in ("asc", "desc"):
                normalized_order = "asc"
                
            # Get the appropriate statement key
            stmt_key = f"{sort_by}_{normalized_order}"
        
        # Use the appropriate prepared statement
        if stmt_key and stmt_key in prepared_statements:
            stmt = prepared_statements[stmt_key]
        else:
            # If sort_by is invalid or not specified, use the default statement
            stmt = default_statement
        
        # Execute the prepared statement and return all contacts
        return session.execute(stmt).scalars().all()
    finally:
        session.close()

@cached_query(cache_type="standard", ttl=120)
def get_contacts_page(
    sort_by: Optional[str] = None, 
    order: Optional[str] = "asc",
    page: int = 1,
    page_size: int = DEFAULT_PAGE_SIZE
) -> List[Contact]:
    """
    Get a specific page of contacts
    
    Args:
        sort_by: Field to sort by
        order: Sort order
        page: Page number (1-based)
        page_size: Number of items per page
        
    Returns:
        List of contacts for the requested page
    """
    # Validate input
    page = max(1, page)  # Page must be at least 1
    page_size = min(max(1, page_size), 100)  # Page size between 1 and 100
    
    session = Session()
    try:
        # Get the paginated statement
        stmt, _, _ = get_paginated_statement(sort_by, order, page, page_size)
        
        # Execute the statement and return the results
        return session.execute(stmt).scalars().all()
    finally:
        session.close()

def get_max_pages(total_contacts: int, page_size: int) -> int:
    """
    Calculate the maximum number of pages
    
    Args:
        total_contacts: Total number of contacts
        page_size: Number of items per page
        
    Returns:
        Maximum number of pages
    """
    return math.ceil(total_contacts / page_size) if total_contacts > 0 else 1

def list_contacts(
    sort_by: Optional[str] = None,
    order: Optional[str] = "asc",
    page: int = typer.Option(
        1, "--page", "-p", help="Page number (1-based)", min=1
    ),
    page_size: int = typer.Option(
        DEFAULT_PAGE_SIZE, "--page-size", "-s", help="Number of contacts per page", min=1, max=100
    ),
    all_pages: bool = typer.Option(
        False, "--all", "-a", help="Show all contacts without pagination"
    )
):
    """
    List contacts with pagination and sorting
    
    This implementation:
    1. Uses efficient pagination to handle large datasets
    2. Includes sorting options
    3. Caches results for better performance
    4. Provides memory-efficient handling of large contact lists
    """
    try:
        # Get total count for pagination
        total_contacts = get_contact_count()
        
        if total_contacts == 0:
            typer.echo("No contacts found.")
            return
            
        # Extract actual values from Typer options
        page_value = page if isinstance(page, int) else 1
        page_size_value = page_size if isinstance(page_size, int) else DEFAULT_PAGE_SIZE
        
        # Validate page number
        max_pages = get_max_pages(total_contacts, page_size_value)
        if page_value > max_pages:
            typer.echo(f"Page {page_value} exceeds maximum pages ({max_pages}). Showing last page.")
            page_value = max_pages
        
        # Handle invalid sort field warning
        if sort_by and sort_by not in SORT_FIELD_MAP:
            typer.echo(f"Warning: Invalid sort field '{sort_by}'. Using default order.")
            sort_by = None
            
        # Get all_pages value
        all_pages_value = all_pages if isinstance(all_pages, bool) else False
        
        # Get contacts (either all or paginated)
        if all_pages_value:
            # Get all contacts if requested
            contacts = _list_contacts_cached(sort_by, order)
        else:
            # Get paginated contacts
            contacts = get_contacts_page(sort_by, order, page_value, page_size_value)
        
        if not contacts:
            typer.echo("No contacts found.")
            return
        
        # Print header
        typer.echo("ID                                    | First Name    | Last Name     | Email")
        typer.echo("-" * 80)
        
        # Print each contact
        for contact in contacts:
            typer.echo(f"{contact.local_id} | {contact.name_first:<13} | {contact.name_last:<13} | {contact.email}")
            
        # Print pagination information
        if not all_pages_value:
            typer.echo(f"\nShowing page {page_value} of {max_pages}")
            typer.echo(f"Records {(page_value-1)*page_size_value+1}-{min(page_value*page_size_value, total_contacts)} of {total_contacts}")
        else:
            typer.echo(f"\nTotal contacts: {total_contacts}")
        
        # Show sorting information if applied
        if sort_by and sort_by in SORT_FIELD_MAP:
            sort_direction = "desc" if order and order.lower() == "desc" else "asc"
            typer.echo(f"Sorted by: {sort_by} ({sort_direction}ending)")
            
        # Show pagination help if not showing all pages
        if not all_pages_value and max_pages > 1:
            typer.echo("\nUse --page to navigate between pages (e.g., --page 2)")
            typer.echo("Use --all to show all contacts at once")
            
    except Exception as e:
        typer.echo(f"Error listing contacts: {str(e)}", err=True)
        raise