"""
Missing email filter and management functionality for ContactCMD.
This module provides functionality to list contacts without email addresses
and a UI to manage them (add email, skip to next, or discard).
"""

import typer
from sqlalchemy import or_
from db.database import Session
from models.core import Contact
from contact import add_contact

def find_contacts_without_email():
    """
    Find all contacts that don't have an email address.
    
    Returns:
        List of Contact objects without email addresses, sorted by name.
    """
    session = Session()
    try:
        # Query contacts with null or empty email
        contacts = session.query(Contact).filter(
            or_(
                Contact.email == None,
                Contact.email == ""
            )
        ).order_by(Contact.name_last, Contact.name_first).all()
        
        return contacts
    finally:
        session.close()

def update_contact_email(contact_id, email):
    """
    Update a contact's email address.
    
    Args:
        contact_id: UUID of the contact to update
        email: New email address
        
    Returns:
        Updated Contact object
    """
    session = Session()
    try:
        contact = session.query(Contact).filter(Contact.local_id == contact_id).first()
        if not contact:
            raise ValueError(f"Contact with ID {contact_id} not found")
        
        contact.email = email
        contact.is_dirty = True  # Mark for sync
        session.commit()
        return contact
    finally:
        session.close()

def list_contacts_without_email():
    """
    List all contacts without email addresses.
    
    This function is used by the CLI commands.
    """
    contacts = find_contacts_without_email()
    
    if not contacts:
        typer.echo("All contacts have email addresses. Nothing to fix! 🎉")
        return []
    
    # Display a formatted table of contacts without emails
    typer.echo(f"Found {len(contacts)} contacts without email addresses:\n")
    
    # Print header with comprehensive information
    typer.echo(f"{'ID':<36} | {'First Name':<15} | {'Last Name':<15} | {'Created At':<19} | {'Sync Status':<12}")
    typer.echo("-" * 100)
    
    # Print each contact with more comprehensive information
    for contact in contacts:
        created_at = contact.created_at.strftime("%Y-%m-%d %H:%M") if contact.created_at else "N/A"
        typer.echo(f"{str(contact.local_id):<36} | "
                  f"{contact.name_first or 'N/A':<15} | "
                  f"{contact.name_last or 'N/A':<15} | "
                  f"{created_at:<19} | "
                  f"{contact.sync_status or 'N/A':<12}")
    
    return contacts

def interactive_email_manager():
    """
    Interactive UI to manage contacts without email addresses.
    
    This function presents each contact without an email and allows the user to:
    1. Add an email address
    2. Skip to the next contact
    3. Discard (stop the process)
    """
    contacts = find_contacts_without_email()
    
    if not contacts:
        typer.echo("All contacts have email addresses. Nothing to fix! 🎉")
        return
    
    total = len(contacts)
    processed = 0
    skipped = 0
    updated = 0
    
    typer.echo(f"Found {total} contacts without email addresses.\n")
    typer.echo("Starting interactive email management session.")
    typer.echo("For each contact, you can add an email, skip to the next one, or exit.\n")
    
    for contact in contacts:
        full_name = contact.name_first or '' 
        if contact.name_last:
            full_name += f" {contact.name_last}"
        
        typer.echo(f"Contact {processed+1}/{total}: {full_name or 'Unknown Name'}")
        typer.echo("=" * 60)
        
        # Basic contact information section
        typer.echo("CONTACT INFORMATION")
        typer.echo("-" * 60)
        typer.echo(f"ID:           {contact.local_id}")
        typer.echo(f"First Name:   {contact.name_first or 'N/A'}")
        typer.echo(f"Last Name:    {contact.name_last or 'N/A'}")
        typer.echo(f"Email:        {contact.email or 'Missing'}")
        
        # Show sync notes if available to diagnose issues
        if hasattr(contact, 'sync_notes') and contact.sync_notes:
            typer.echo(f"Sync Notes:   {contact.sync_notes}")
        
        # The Contact model doesn't have address and phone fields yet,
        # so we'll just show what's available for now
        typer.echo("\nSYSTEM INFORMATION")
        typer.echo("-" * 60)
        typer.echo(f"Created At:   {contact.created_at.strftime('%Y-%m-%d %H:%M') if contact.created_at else 'N/A'}")
        typer.echo(f"Updated At:   {contact.updated_at.strftime('%Y-%m-%d %H:%M') if contact.updated_at else 'N/A'}")
        typer.echo(f"Sync Status:  {contact.sync_status or 'N/A'}")
        typer.echo(f"Is Dirty:     {contact.is_dirty}")
        if contact.last_synced_at:
            typer.echo(f"Last Synced:  {contact.last_synced_at.strftime('%Y-%m-%d %H:%M')}")
        typer.echo("=" * 60)
        
        while True:
            choice = typer.prompt(
                "Options: [a]dd email, [s]kip, [d]iscard/exit",
                default="a"
            ).lower()
            
            if choice in ('d', 'discard', 'exit'):
                typer.echo("\nExiting email management session.")
                typer.echo(f"Updated: {updated}, Skipped: {skipped}, Remaining: {total - processed}")
                return
            
            elif choice in ('s', 'skip'):
                skipped += 1
                processed += 1
                typer.echo("Skipped to next contact.\n")
                break
                
            elif choice in ('a', 'add'):
                email = typer.prompt("Enter email address")
                
                if not email.strip():
                    typer.echo("Email cannot be empty. Try again or skip.")
                    continue
                    
                try:
                    update_contact_email(contact.local_id, email)
                    updated += 1
                    processed += 1
                    typer.echo(f"Email updated to: {email}\n")
                    break
                except Exception as e:
                    typer.echo(f"Error updating email: {e}")
            
            else:
                typer.echo("Invalid option. Try again.")
    
    # Finished processing all contacts
    typer.echo("\nCompleted email management session.")
    typer.echo(f"Updated: {updated}, Skipped: {skipped}, Total processed: {processed}")
    typer.echo("All contacts have been processed. 🎉")