import typer
from db.database import Session
from models import Contact
from sqlalchemy import or_, select, bindparam, func
from db.cache import cached_query
from typing import List, Optional
import os
import importlib.util

# Create high-performance prepared statements at module level
# Complex searches are split to optimize SQLite index usage
# - Prefix searches are more index-friendly than wildcard searches

# First name prefix search
first_name_prefix_stmt = select(Contact).where(
    Contact.name_first.ilike(bindparam('first_name_prefix') + '%')
)

# Last name prefix search
last_name_prefix_stmt = select(Contact).where(
    Contact.name_last.ilike(bindparam('last_name_prefix') + '%')
)

# Email prefix search (very common pattern)
email_prefix_stmt = select(Contact).where(
    Contact.email.ilike(bindparam('email_prefix') + '%')
)

# Full text search (fallback for non-prefix searches)
# Create prepared statement for case-insensitive search
case_insensitive_search_stmt = select(Contact).where(
    or_(
        Contact.name_first.ilike(bindparam('query_pattern')),
        Contact.name_last.ilike(bindparam('query_pattern')),
        Contact.email.ilike(bindparam('query_pattern'))
    )
)

# Create prepared statement for case-sensitive search
case_sensitive_search_stmt = select(Contact).where(
    or_(
        Contact.name_first.like(bindparam('query_pattern')),
        Contact.name_last.like(bindparam('query_pattern')),
        Contact.email.like(bindparam('query_pattern'))
    )
)

# Check if Cython module is available
CYTHON_AVAILABLE = False
try:
    # Try to import compiled Cython module
    from . import search_cy
    CYTHON_AVAILABLE = True
except ImportError:
    # Check if we have the .pyx file but it's not compiled
    file_path = os.path.join(os.path.dirname(__file__), "search_cy.pyx")
    if os.path.exists(file_path):
        # We have the source but not the compiled version
        typer.echo("Cython search module found but not compiled. Using Python implementation.")
    CYTHON_AVAILABLE = False

@cached_query(cache_type="search", ttl=120)
def _search_contacts_optimized(query: str, case_sensitive: bool = False) -> List[Contact]:
    """
    Optimized contact search function with adaptive caching
    
    Args:
        query: Search query string
        case_sensitive: Whether to use case-sensitive search
        
    Returns:
        List of matching Contact objects
    """
    session = Session()
    try:
        # Detect if this is a prefix search (no wildcards) for optimization
        is_prefix_search = '%' not in query and '*' not in query and '?' not in query
        
        # For prefix searches, use optimized prefix-specific prepared statements
        # that leverage indexes more effectively
        if is_prefix_search:
            # Try first name prefix
            first_results = session.execute(
                first_name_prefix_stmt.params(first_name_prefix=query)
            ).scalars().all()
            
            # Try last name prefix
            last_results = session.execute(
                last_name_prefix_stmt.params(last_name_prefix=query)
            ).scalars().all()
            
            # Try email prefix (common search pattern)
            email_results = session.execute(
                email_prefix_stmt.params(email_prefix=query)
            ).scalars().all()
            
            # Combine results, removing duplicates (using set)
            all_ids = set()
            all_contacts = []
            
            # Process results from most specific to least specific
            for contact in first_results + last_results + email_results:
                if str(contact.local_id) not in all_ids:
                    all_ids.add(str(contact.local_id))
                    all_contacts.append(contact)
                    
            return all_contacts
        else:
            # For wildcard searches, use the full pattern search
            query_pattern = query if '%' in query else f"%{query}%"
            
            # Select the appropriate prepared statement
            stmt = case_sensitive_search_stmt if case_sensitive else case_insensitive_search_stmt
            
            # Bind the parameter to the prepared statement
            bound_stmt = stmt.params(query_pattern=query_pattern)
            
            # Execute the prepared statement
            return session.execute(bound_stmt).scalars().all()
    finally:
        session.close()

def search_contacts(
    query: str = typer.Argument(..., help="Search query to match against contacts"),
    case_sensitive: bool = typer.Option(
        False, "--case-sensitive", "-c", help="Make search case sensitive"
    ),
    page: int = typer.Option(
        1, "--page", "-p", help="Page number (1-based)", min=1
    ),
    page_size: int = typer.Option(
        20, "--page-size", "-s", help="Number of contacts per page", min=1, max=100
    ),
    all_results: bool = typer.Option(
        False, "--all", "-a", help="Show all search results without pagination"
    )
):
    """
    Search for contacts by name or email using optimized prepared statements and caching
    
    This implementation:
    1. Uses specialized prepared statements for prefix vs wildcard searches
    2. Caches search results for better performance with repeated searches
    3. Uses index-friendly query patterns where possible
    4. Provides better performance for large contact databases
    5. Uses Cython implementation when available for maximum performance
    6. Supports pagination for large result sets
    """
    try:
        # Use Cython implementation if available
        if CYTHON_AVAILABLE:
            # The Cython implementation handles displaying results as well
            # But we'll wrap it to add pagination support
            all_contacts = search_cy.search_contacts_cy(query, case_sensitive, display_results=False)
        else:
            # Fall back to Python implementation
            all_contacts = _search_contacts_optimized(query, case_sensitive)
        
        if not all_contacts:
            typer.echo(f"No contacts found matching '{query}'.")
            return
        
        # Get total count
        total_results = len(all_contacts)
        
        # Calculate max pages
        max_pages = (total_results + page_size - 1) // page_size
        
        # Validate page number
        if page > max_pages:
            typer.echo(f"Page {page} exceeds maximum pages ({max_pages}). Showing last page.")
            page = max_pages
        
        # Apply pagination if not showing all results
        if not all_results:
            # Calculate slice indices
            start_idx = (page - 1) * page_size
            end_idx = min(start_idx + page_size, total_results)
            
            # Slice the results
            contacts = all_contacts[start_idx:end_idx]
        else:
            contacts = all_contacts
        
        # Print header
        typer.echo(f"Search results for '{query}':")
        typer.echo("ID                                    | First Name    | Last Name     | Email")
        typer.echo("-" * 80)
        
        # Print each contact
        for contact in contacts:
            typer.echo(f"{contact.local_id} | {contact.name_first:<13} | {contact.name_last:<13} | {contact.email}")
        
        # Print pagination information
        if not all_results and max_pages > 1:
            typer.echo(f"\nShowing page {page} of {max_pages}")
            typer.echo(f"Results {(page-1)*page_size+1}-{min(page*page_size, total_results)} of {total_results}")
        else:
            # Print total count
            typer.echo(f"\nFound {total_results} matching contacts")
            
        # Show pagination help if not showing all pages and we have multiple pages
        if not all_results and max_pages > 1:
            typer.echo("\nUse --page to navigate between pages (e.g., --page 2)")
            typer.echo("Use --all to show all results at once")
            
    except Exception as e:
        typer.echo(f"Error searching contacts: {str(e)}", err=True)
        raise