# cython: language_level=3str, boundscheck=False, wraparound=False, initializedcheck=False, cdivision=True, binding=True, optimize.use_switch=True
# Highly optimized Cython version of search module with static typing

import typer
from sqlalchemy import or_, select
from sqlalchemy.sql.expression import bindparam
import cython
from libc.stdlib cimport malloc, free
from cpython cimport array
import array

from db.database import Session
from models.core import Contact
from db.cache import cached_query

# Cython type declarations for improved performance
from cpython.ref cimport PyObject
from cpython.list cimport PyList_Append, PyList_GET_SIZE, PyList_GET_ITEM, PyList_New
from cpython.set cimport PySet_Add, PySet_Contains

# Create optimized prepared statements at module level
# First name prefix search
first_name_prefix_stmt = select(Contact).where(
    Contact.name_first.ilike(bindparam('first_name_prefix') + '%')
)

# Last name prefix search
last_name_prefix_stmt = select(Contact).where(
    Contact.name_last.ilike(bindparam('last_name_prefix') + '%')
)

# Email prefix search (very common pattern)
email_prefix_stmt = select(Contact).where(
    Contact.email.ilike(bindparam('email_prefix') + '%')
)

# Full text search statements
case_insensitive_search_stmt = select(Contact).where(
    or_(
        Contact.name_first.ilike(bindparam('query_pattern')),
        Contact.name_last.ilike(bindparam('query_pattern')),
        Contact.email.ilike(bindparam('query_pattern'))
    )
)

case_sensitive_search_stmt = select(Contact).where(
    or_(
        Contact.name_first.like(bindparam('query_pattern')),
        Contact.name_last.like(bindparam('query_pattern')),
        Contact.email.like(bindparam('query_pattern'))
    )
)

# Cache for search results with optimized TTL
@cached_query(cache_type="search", ttl=120)
def _search_contacts_cy_cached(str query, bint case_sensitive=False):
    """
    Cached implementation of the optimized Cython search
    
    Uses the 'search' cache type for optimized TTL settings
    """
    return _search_contacts_optimized_cy(query, case_sensitive)

@cython.boundscheck(False)
@cython.wraparound(False)
cdef list _search_contacts_optimized_cy(str query, bint case_sensitive=False):
    """
    Ultra-optimized Cython implementation of contact search
    """
    cdef:
        object session = Session()
        list results = []
        set all_ids = set()
        object contact
        bint is_prefix_search
        str query_pattern
        object stmt, bound_stmt
        list first_results, last_results, email_results
    
    try:
        # Detect if this is a prefix search (no wildcards) for optimization
        is_prefix_search = '%' not in query and '*' not in query and '?' not in query
        
        # For prefix searches, use optimized prefix-specific prepared statements
        if is_prefix_search:
            # First name prefix search (optimized)
            first_results = list(session.execute(
                first_name_prefix_stmt.params(first_name_prefix=query)
            ).scalars().all())
            
            # Last name prefix search (optimized)
            last_results = list(session.execute(
                last_name_prefix_stmt.params(last_name_prefix=query)
            ).scalars().all())
            
            # Email prefix search (optimized)
            email_results = list(session.execute(
                email_prefix_stmt.params(email_prefix=query)
            ).scalars().all())
            
            # High-performance duplicate removal
            for contact in first_results + last_results + email_results:
                contact_id = str(contact.local_id)
                if contact_id not in all_ids:
                    all_ids.add(contact_id)
                    PyList_Append(results, contact)
            
            return results
        else:
            # For wildcard searches, use the full pattern search
            query_pattern = query if '%' in query else f"%{query}%"
            
            # Select the appropriate prepared statement
            stmt = case_sensitive_search_stmt if case_sensitive else case_insensitive_search_stmt
            
            # Execute with parameter binding
            bound_stmt = stmt.params(query_pattern=query_pattern)
            return list(session.execute(bound_stmt).scalars().all())
    except Exception as e:
        typer.echo(f"Error in Cython search: {str(e)}", err=True)
        raise
    finally:
        session.close()

def search_contacts_cy(str query, bint case_sensitive=False, bint display_results=True):
    """
    Cythonized search for contacts by name or email using prepared statements
    
    Args:
        query: Search query to match against contacts
        case_sensitive: Whether to make search case sensitive
        display_results: Whether to display results (default: True)
    
    Returns:
        List of matching Contact objects
    """
    # Use cached implementation for better performance
    contacts = _search_contacts_cy_cached(query, case_sensitive)
    
    if not contacts:
        if display_results:
            typer.echo(f"No contacts found matching '{query}'.")
        return []
    
    # Display results only if requested
    if display_results:
        _display_search_results(query, contacts)
    
    # Return for potential further processing
    return contacts

# Optimized display function
@cython.boundscheck(False)
@cython.wraparound(False)
cdef void _display_search_results(str query, list contacts):
    """Display search results in a formatted table"""
    cdef:
        object contact
        int count = len(contacts)
        int i
    
    typer.echo(f"Search results for '{query}':")
    typer.echo("ID                                    | First Name    | Last Name     | Email")
    typer.echo("-" * 80)
    
    # Faster iteration
    for i in range(count):
        contact = contacts[i]
        typer.echo(f"{contact.local_id} | {contact.name_first:<13} | {contact.name_last:<13} | {contact.email}")
        
    # Print total count
    typer.echo(f"\nFound {count} matching contacts")

# Wrapper function to maintain compatibility with existing code
def search_contacts(query, case_sensitive=False):
    """
    Search for contacts by name or email
    
    Wrapper around the Cythonized search function for backward compatibility
    """
    return search_contacts_cy(query, case_sensitive)