import uuid
import typer
from sqlalchemy import select, bindparam
from sqlalchemy.orm import joinedload
from db.database import Session
from models import Contact
from db.cache import cached_query

# Prepared statement for contact lookup with eager loading of relationships
contact_stmt = (
    select(Contact)
    .options(
        joinedload(Contact.interactions),
        joinedload(Contact.special_dates)
    )
    .where(Contact.local_id == bindparam('contact_id'))
)

@cached_query(ttl=30, key_prefix="contact_details")
def get_contact_with_related(contact_uuid):
    """
    Get a contact with all related data in a single query (cached for 30 seconds)
    """
    with Session() as session:
        stmt = contact_stmt.params(contact_id=contact_uuid)
        result = session.execute(stmt).scalars().first()
        return result

def show_contact(contact_id: str):
    """
    Show details of a specific contact with caching
    """
    try:
        # Convert string to UUID
        try:
            contact_uuid = uuid.UUID(contact_id)
        except ValueError:
            typer.echo(f"Invalid UUID format: {contact_id}", err=True)
            return
        
        # Get contact using the cached function
        contact = get_contact_with_related(contact_uuid)
        
        if not contact:
            typer.echo(f"Contact with ID {contact_id} not found.")
            return
        
        # Print contact details
        typer.echo(f"Contact Details:")
        typer.echo(f"ID:           {contact.local_id}")
        typer.echo(f"First Name:   {contact.name_first}")
        typer.echo(f"Last Name:    {contact.name_last}")
        typer.echo(f"Email:        {contact.email}")
        typer.echo(f"Created At:   {contact.created_at}")
        typer.echo(f"Updated At:   {contact.updated_at}")
        typer.echo(f"Sync Status:  {contact.sync_status}")
        typer.echo(f"Is Dirty:     {contact.is_dirty}")
        if contact.last_synced_at:
            typer.echo(f"Last Synced:  {contact.last_synced_at}")
            
        # Print related data counts
        interaction_count = len(contact.interactions)
        special_date_count = len(contact.special_dates)
        
        if interaction_count > 0:
            typer.echo(f"\nInteractions:  {interaction_count}")
        
        if special_date_count > 0:
            typer.echo(f"Special Dates: {special_date_count}")
            
    except Exception as e:
        typer.echo(f"Error showing contact: {str(e)}", err=True)
        raise