import typer
import cmd
import shlex
import os
import sys
import traceback
import readline  # Import readline for tab completion
from click.exceptions import UsageError

def preload_database():
    """
    Preload database for faster first query
    
    This function optimizes startup performance by:
    1. Pre-loading SQLAlchemy metadata and table definitions
    2. Initializing the connection pool early
    3. Warming up the SQLAlchemy ORM mappers
    4. Pre-executing a simple query to prepare SQLite
    
    Performance impact:
    - Reduces first-query latency (cold start effect)
    - Moves expensive operations to startup instead of first command
    - Makes all commands feel more responsive to users
    """
    try:
        # Import here to avoid circular imports
        from sqlalchemy import text
        from db.database import engine, Base, Session
        from models.core import Contact
        
        # Touch the metadata to load it
        Base.metadata.reflect(bind=engine)
        
        # Initialize a connection in the pool and prime the ORM
        session = Session()
        # Execute a simple query to warm up the connection and mappers
        session.execute(text("SELECT 1")).scalar()
        session.close()
    except ImportError as e:
        # Handle import errors (possible during initial setup or if deps missing)
        # Silent handling is OK for preloading since this is just an optimization
        pass
    except Exception as e:
        # Log to stderr but continue - preloading is a performance optimization, not critical
        # This prevents issues if database isn't ready
        print(f"Database preloading warning: {e}", file=sys.stderr)
        # Uncomment for debugging: traceback.print_exc(file=sys.stderr)

# Preload database to speed up startup
preload_database()

# Create the main Typer app
app = typer.Typer(
    help="""
    Contact management CLI for tracking contacts, interactions, and special dates.
    
    ContactCMD is a high-performance CLI tool that stores all data locally in a SQLite database.
    It allows you to manage contacts, record interactions, track important dates, and sync with
    external systems like macOS Contacts.
    
    🚀 QUICK START:
      contactcmd                     - Start interactive shell (recommended for beginners)
      contactcmd contact add         - Add a new contact with guided prompts
      contactcmd contact list        - Show all your contacts
      contactcmd help               - Get detailed help
    
    💡 USAGE PATTERNS:
      contactcmd [COMMAND] [OPTIONS]
      contactcmd                     - Interactive shell mode with tab completion
      contactcmd contact add --name-first John --name-last Doe --email john@example.com
      contactcmd nl "add jane smith with phone 555-1234"  - Natural language commands
    
    📋 MAIN COMMANDS:
      contact                        - Manage contacts (add, list, search, delete)
      interaction                    - Record interactions with contacts
      special-date                   - Track birthdays, anniversaries, etc.
      sync                           - Sync with external sources (macOS Contacts)
      llm                           - Manage AI models for natural language features
      nl                            - Use natural language commands
      
    🎯 INTERACTIVE SHELL:
      Run 'contactcmd' without arguments to enter interactive shell mode
      • Tab completion for commands and contact names
      • Command history with arrow keys
      • Natural language processing (if enabled)
      • Type 'help' for all commands or 'exit' to quit
      
    📚 DOCUMENTATION:
      contactcmd --help              - Show this help message
      contactcmd <command> --help    - Show help for a specific command
      
    🤖 NATURAL LANGUAGE:
      First run will offer to download a local AI model (669 MB) for features like:
      • "show contacts added this week"
      • "find people without email addresses"
      • "add meeting notes for john doe"
    """,
    add_completion=True  # Enable shell completion support
)

# Create subcommand apps for each area
contact_app = typer.Typer(
    name="contact", 
    help="""
    Manage contacts - add, list, show details, search, and delete.
    
    Use this command to manage your contacts database. You can add contacts,
    list them, search for specific ones, and perform other management tasks.
    
    EXAMPLES:
      contactcmd contact add --name-first "John" --name-last "Doe" --email "john.doe@example.com"
      contactcmd contact list --sort-by last_name
      contactcmd contact search "doe"
      contactcmd contact missing-email
      contactcmd contact fix-missing-emails
    """,
    add_completion=True
)
interaction_app = typer.Typer(
    name="interaction", 
    help="""
    Manage interactions with contacts - record calls, meetings, notes, etc.
    
    Use this command to log various types of interactions with your contacts,
    such as calls, meetings, emails, and general notes.
    
    EXAMPLES:
      contactcmd interaction add <contact_id> --type "call" --notes "Discussed project timeline"
    """,
    add_completion=True
)
special_date_app = typer.Typer(
    name="special-date", 
    help="""
    Manage special dates for contacts - birthdays, anniversaries, etc.
    
    Use this command to track important dates related to your contacts,
    such as birthdays, anniversaries, and other custom date types.
    
    EXAMPLES:
      contactcmd special-date add <contact_id> --type "birthday" --date "1990-05-15" --description "John's birthday"
    """,
    add_completion=True
)
sync_app = typer.Typer(
    name="sync",
    help="""
    Sync contacts with external sources like macOS Contacts.
    
    Use this command to synchronize your contacts with external systems,
    currently supporting synchronization with macOS Contacts app.
    
    EXAMPLES:
      contactcmd sync mac
      contactcmd sync mac --all --dry-run
    """,
    add_completion=True
)

llm_app = typer.Typer(
    name="llm",
    help="""
    Manage Local Language Models (LLMs) for natural language processing.
    
    Use this command to download, manage, and configure LLM models that enable
    natural language understanding of your commands.
    
    EXAMPLES:
      contactcmd llm list              - List available and installed models
      contactcmd llm download tinyllama - Download the TinyLlama model
      contactcmd llm enable            - Enable LLM features
      contactcmd llm status            - Check LLM configuration status
    """,
    add_completion=True
)

# Add the subcommand apps to the main app
app.add_typer(contact_app)
app.add_typer(interaction_app)
app.add_typer(special_date_app)
app.add_typer(sync_app)
app.add_typer(llm_app)

@contact_app.command("add")
def contact_add(
    name_first: str = typer.Option(..., "--name-first", help="First name of the contact"),
    name_last: str = typer.Option(..., "--name-last", help="Last name of the contact"),
    email: str = typer.Option(..., "--email", help="Email address of the contact")
):
    """
    Add a new contact to the database.
    
    This command adds a new contact with the provided information. All three fields
    (first name, last name, and email) are required. Upon success, the command will
    return the UUID of the new contact, which you'll need for other operations.
    
    Example usage:
        contactcmd contact add --name-first "John" --name-last "Doe" --email "john.doe@example.com"
        contactcmd contact add --name-first "Jane" --name-last "Smith" --email "jane.smith@company.com"
        
    Notes:
        - The contact ID is automatically generated
        - You'll need this ID for interactions and special dates
        - Email addresses don't need to be unique, but it's recommended
    """
    from contact import add_contact
    return add_contact(name_first=name_first, name_last=name_last, email=email)

@contact_app.command("list")
def contact_list(
    sort_by: str = typer.Option(
        None, "--sort-by", "-s", help="Field to sort by (first_name, last_name, email, created)"
    ),
    order: str = typer.Option(
        "asc", "--order", "-o", help="Sort order (asc, desc)"
    ),
):
    """
    List all contacts with optional sorting.
    
    This command displays all contacts in the database. You can sort the results
    by different fields and specify the sort order.
    
    Examples:
        contact list
        contact list --sort-by last_name
        contact list --sort-by email --order desc
    """
    from contact import list_contacts
    return list_contacts(sort_by=sort_by, order=order)

@contact_app.command("show")
def contact_show(contact_id: str):
    """
    Show details of a specific contact.
    
    This command displays all available information about a contact,
    including their UUID, name, email, creation date, and sync status.
    
    Example:
        contact show 12345678-1234-5678-1234-567812345678
    """
    from contact import show_contact
    return show_contact(contact_id)

@contact_app.command("search")
def contact_search(
    query: str,
    case_sensitive: bool = typer.Option(
        False, "--case-sensitive", "-c", help="Make search case sensitive"
    ),
    page: int = typer.Option(
        1, "--page", "-p", help="Page number (1-based)", min=1
    ),
    page_size: int = typer.Option(
        20, "--page-size", "-s", help="Number of contacts per page", min=1, max=100
    ),
    all_results: bool = typer.Option(
        False, "--all", "-a", help="Show all search results without pagination"
    ),
):
    """
    Search for contacts by name or email.
    
    This command searches across first name, last name, and email fields.
    The search is case-insensitive by default but can be made case-sensitive.
    
    Examples:
        contact search "doe"
        contact search "GMAIL" --case-sensitive
        contact search "doe" --page 2 --page-size 10
        contact search "doe" --all
    """
    from contact import search_contacts
    return search_contacts(query=query, case_sensitive=case_sensitive, 
                          page=page, page_size=page_size, all_results=all_results)

@contact_app.command("delete")
def contact_delete(contact_id: str):
    """
    Delete a specific contact by ID.
    
    This command permanently removes a contact from the database.
    You will be asked to confirm by typing 'DELETE' in all uppercase.
    This action will cascade to all backups when sync happens.
    
    Example:
        contact delete 12345678-1234-5678-1234-567812345678
    """
    from contact import delete_contact
    return delete_contact(contact_id)

@contact_app.command("delete-all")
def contact_delete_all():
    """
    Delete ALL contacts from the database.
    
    This command permanently removes ALL contacts from the database.
    You will be asked to confirm by typing 'DELETE' in all uppercase.
    This action will cascade to all backups when sync happens and CANNOT BE UNDONE.
    Use with extreme caution.
    
    Example:
        contact delete-all
    """
    from contact import delete_all_contacts
    return delete_all_contacts()

@contact_app.command("missing-email")
def contact_missing_email():
    """
    List all contacts that don't have an email address.
    
    This command displays a list of all contacts in the database that 
    don't have an email address assigned. These contacts may have been 
    imported from external sources or added manually without an email.
    
    The contacts are displayed in a tabular format with their ID, first name,
    and last name to help you identify them.
    
    Example:
        contact missing-email
    """
    from contact.missing_email import list_contacts_without_email
    return list_contacts_without_email()

@contact_app.command("fix-missing-emails")
def contact_fix_missing_emails():
    """
    Interactive UI to add email addresses to contacts missing them.
    
    This command starts an interactive session to help you add email addresses
    to contacts that don't have them. For each contact without an email address,
    you can:
    
    1. Add an email address
    2. Skip to the next contact
    3. Discard the process entirely
    
    The command keeps track of how many contacts you've updated, skipped,
    and how many remain.
    
    Example:
        contact fix-missing-emails
    """
    from contact.missing_email import interactive_email_manager
    return interactive_email_manager()

@contact_app.command("missing-names")
def contact_missing_names():
    """
    List all contacts that don't have a first and last name.
    
    This command displays a list of all contacts in the database that
    don't have both first and last names assigned. These contacts may have been
    imported incorrectly from external sources.
    
    The contacts are displayed in a tabular format with their ID, email,
    and creation date to help you identify them.
    
    Example:
        contact missing-names
    """
    from contact.fix_names import list_contacts_with_missing_names
    return list_contacts_with_missing_names()

@contact_app.command("fix-missing-names")
def contact_fix_missing_names():
    """
    Attempt to fix contacts with missing names.
    
    This command will try to repair contacts that are missing names by:
    1. Finding contacts with empty first and last names
    2. Looking up their Apple ID in sync_notes (if they were synced from macOS)
    3. Fetching the correct name from macOS Contacts
    4. Updating the database with the correct name information
    
    This command requires macOS and access to the Contacts app.
    
    Example:
        contact fix-missing-names
    """
    from contact.fix_names import fix_missing_names
    return fix_missing_names()

# Wire up interaction commands
@interaction_app.command("add")
def interaction_add(
    contact_id: str,
    type: str = typer.Option(..., "--type", help="Type of interaction (e.g., note, call, meeting)"),
    text: str = typer.Option(..., "--notes", help="Notes about the interaction")
):
    """
    Add a new interaction for a contact.
    
    This command records an interaction with a specific contact.
    You need to provide the contact's UUID, the type of interaction,
    and notes about what happened.
    
    Interaction types can include: note, call, meeting, email
    
    Example:
        interaction add 12345678-1234-5678-1234-567812345678 --type "call" --notes "Discussed project timeline"
    """
    from interaction import add_interaction
    return add_interaction(contact_id=contact_id, type=type, text=text)

# Wire up special date commands

# Wire up sync commands

@sync_app.command("debug")
def sync_debug_cmd():
    """Check sync compatibility and configuration."""
    from sync.simple import check_platform
    return check_platform()

@sync_app.command("mac")
def sync_mac_cmd(
    all_contacts: bool = typer.Option(False, "--all", "-a", help="Sync all contacts, not just recent changes"),
    dry_run: bool = typer.Option(False, "--dry-run", "-d", help="Don't make any changes, just show what would happen")
):
    """
    Sync contacts from macOS Contacts app to contactcmd.
    
    EXAMPLE USAGE:
      python -m contactcmd sync mac
      python -m contactcmd sync mac --all
      python -m contactcmd sync mac --dry-run
      python -m contactcmd sync mac --debug
    
    DESCRIPTION:
      This command imports contacts from your Mac's built-in Contacts application
      into contactcmd. It creates a link between Mac contacts and contactcmd
      contacts for reliable future syncing.
    
    OPTIONS:
      --all, -a     Import all contacts regardless of when they were last modified
                    (without this flag, only contacts changed since the last sync
                    will be imported)
      --dry-run, -d Show what changes would be made without actually making them
                    (useful for previewing the sync operation)
      --debug       Enable debug logging for troubleshooting sync issues
    
    NOTES:
      - First-time use will request permission to access your Mac Contacts
      - Syncs basic contact data: names, emails, and birthdays
      - Creates new contacts in contactcmd and updates existing ones
      - One-way sync only (Mac → contactcmd, not contactcmd → Mac)
      - Stores the sync date to optimize future operations
    
    See docs/MAC_SYNC.md for complete documentation.
    """
    # Import here to avoid importing Mac-specific modules on non-Mac platforms
    import platform
    if platform.system() == 'Darwin':
        from sync.mac import sync_mac_contacts
        # Hard-coded debug=False for now until we figure out the issue with the debug flag
        return sync_mac_contacts(all_contacts=all_contacts, dry_run=dry_run, debug=False)
    else:
        import typer
        typer.echo("Error: macOS contacts sync is only available on macOS")
        return None

@sync_app.command("test")
def sync_test_cmd():
    """Test the sync functionality without accessing macOS APIs."""
    from sync.mac import test_command
    return test_command()

@special_date_app.command("add")
def special_date_add(
    contact_id: str,
    type: str = typer.Option(..., "--type", help="Type of special date (e.g., birthday, anniversary)"),
    date: str = typer.Option(..., "--date", help="Date in YYYY-MM-DD format"),
    description: str = typer.Option("", "--description", help="Optional description for the special date")
):
    """
    Add a new special date for a contact.
    
    This command records an important date associated with a contact, such as a
    birthday or anniversary. You need to provide the contact's UUID, the type of
    date, and the date itself in YYYY-MM-DD format.
    
    Examples:
        contactcmd special-date add 12345678-1234-5678-1234-567812345678 --type "birthday" --date "1990-05-15"
        contactcmd special-date add 12345678-1234-5678-1234-567812345678 --type "anniversary" --date "2020-06-12" --description "Work anniversary"
    
    Special date types:
        - birthday: Person's date of birth
        - anniversary: Work, relationship, or other milestone anniversary
        - reminder: Any other date to remember
        - [custom]: You can use your own custom date types
        
    Notes:
        - Date must be in YYYY-MM-DD format
        - The description field is optional but recommended for clarity
        - You can find a contact's ID using the 'contact list' or 'contact search' commands
    """
    from special_date import add_special_date
    return add_special_date(contact_id=contact_id, type=type, date=date, description=description)

# Wire up LLM commands
@llm_app.command("list")
def llm_list():
    """
    List available and installed LLM models.
    
    Shows all models in the registry with their sizes and descriptions,
    and indicates which models are currently installed.
    """
    from llm.manager import MODEL_REGISTRY, list_installed_models, get_models_path
    
    print("📦 Available Models:")
    print("-" * 70)
    
    installed_models = list_installed_models()
    installed_filenames = {m[1].name for m in installed_models}
    
    for model_id, info in MODEL_REGISTRY.items():
        status = "✅ Installed" if info["filename"] in installed_filenames else "⬇️  Available"
        print(f"\n{model_id}: {info['name']} [{info['size']}] {status}")
        print(f"  {info['description']}")
    
    if installed_models:
        print(f"\n📂 Installed models location: {get_models_path()}")
        total_size = sum(m[2] for m in installed_models)
        print(f"💾 Total disk usage: {total_size:.1f} MB")

@llm_app.command("download")
def llm_download(
    model: str = typer.Argument(..., help="Model ID to download (e.g., tinyllama, phi-2, mistral-7b)")
):
    """
    Download a specific LLM model.
    
    Downloads the specified model from HuggingFace to your local models directory.
    Models are verified with SHA256 checksums to ensure integrity.
    
    Examples:
        contactcmd llm download tinyllama    # Download smallest/fastest model
        contactcmd llm download phi-2        # Download balanced model
        contactcmd llm download mistral-7b   # Download largest/best model
    """
    from llm.manager import download_model
    
    if download_model(model):
        print("\n💡 To use this model, run: contactcmd llm set-default " + model)

@llm_app.command("set-default")
def llm_set_default(
    model: str = typer.Argument(..., help="Model ID or filename to set as default")
):
    """
    Set the default LLM model.
    
    Sets which model will be used for natural language processing.
    You can specify either a model ID (e.g., tinyllama) or the full filename.
    """
    from llm.manager import MODEL_REGISTRY, set_default_model, list_installed_models
    
    # Check if model ID provided
    if model in MODEL_REGISTRY:
        filename = MODEL_REGISTRY[model]["filename"]
    else:
        filename = model
    
    # Verify model is installed
    installed = list_installed_models()
    if not any(m[1].name == filename for m in installed):
        print(f"Error: Model '{model}' is not installed")
        print("Run 'contactcmd llm list' to see installed models")
        return
    
    set_default_model(filename)

@llm_app.command("enable")
def llm_enable():
    """Enable LLM features for natural language command processing."""
    from llm.manager import enable_llm
    enable_llm(True)
    print("💡 You can now use natural language commands!")
    print("Example: contactcmd nl 'show me contacts named John'")

@llm_app.command("disable") 
def llm_disable():
    """Disable LLM features."""
    from llm.manager import enable_llm
    enable_llm(False)

@llm_app.command("status")
def llm_status():
    """Check LLM configuration and status."""
    from llm.manager import check_llm_status
    
    status = check_llm_status()
    
    print("🤖 LLM Status")
    print("-" * 40)
    print(f"Enabled: {'✅ Yes' if status['enabled'] else '❌ No'}")
    print(f"Default model: {status['default_model'] or 'Not set'}")
    
    if status['default_model']:
        if status['default_model_exists']:
            print(f"Default model status: ✅ Ready")
        else:
            print(f"Default model status: ❌ Not found")
    
    print(f"\nModels directory: {status['models_path']}")
    
    if status['installed_models']:
        print(f"\nInstalled models ({len(status['installed_models'])}):")
        for model_id, path, size_mb in status['installed_models']:
            print(f"  - {path.name} ({size_mb:.1f} MB)")
    else:
        print("\nNo models installed yet.")
        print("Run 'contactcmd llm download tinyllama' to get started")

# Add completion command
@app.callback()
def callback():
    """
    Contact management CLI for tracking contacts, interactions, and special dates.
    """
    pass

# Command to explicitly install shell completion
@app.command()
def install_completion(shell: str = typer.Option(None, help="Shell to install completion for (bash, zsh, fish)")):
    """
    Install shell completion for contactcmd.
    
    This command installs shell completion for the specified shell.
    If no shell is specified, it will attempt to detect your current shell.
    
    Supported shells: bash, zsh, fish
    
    Example:
        contactcmd install-completion
        contactcmd install-completion --shell bash
    """
    typer.echo("Installing shell completion...")
    
    # When this command is run, Typer will handle the completion installation
    # based on the --shell parameter
    
    typer.echo(
        "Completion installed. You may need to restart your shell or source "
        "your shell profile file to activate it."
    )

# Natural language command
@app.command("nl")
def natural_language(
    query: str = typer.Argument(..., help="Natural language query")
):
    """
    Process natural language commands using LLM.
    
    This command allows you to use natural language to interact with ContactCMD.
    Requires LLM to be enabled and a model to be downloaded.
    
    Examples:
        contactcmd nl "show me all contacts named John"
        contactcmd nl "find contacts without email addresses"
        contactcmd nl "add a contact named Jane Smith with email jane@example.com"
    """
    from llm.manager import get_config, get_models_path
    from nl_handler import NLCommandHandler
    import os
    
    config = get_config()
    
    # Check if LLM is enabled
    if not config.get("enable_llm", False):
        print("❌ LLM features are not enabled.")
        print("Run 'contactcmd llm enable' to enable natural language processing.")
        return
    
    # Check if default model exists
    default_model = config.get("default_llm")
    if not default_model:
        print("❌ No default LLM model set.")
        print("Run 'contactcmd llm download tinyllama' to download a model.")
        return
    
    model_path = get_models_path() / default_model
    if not model_path.exists():
        print(f"❌ Default model '{default_model}' not found.")
        print("Run 'contactcmd llm list' to see available models.")
        return
    
    # Process the natural language query
    try:
        handler = NLCommandHandler(model_path=str(model_path))
        if not handler.llm_available:
            print("❌ Failed to initialize LLM.")
            return
            
        print(f"🤖 Processing: {query}")
        result = handler.process_command(query)
        
        if result:
            print(f"📋 Executing: {result}")
            # Execute the generated command
            # Note: In a real implementation, you'd parse and execute the command
            # For now, just show what would be executed
        else:
            print("❌ Could not understand the command. Please try rephrasing.")
            
    except Exception as e:
        print(f"❌ Error processing natural language: {e}")

# Define the interactive shell class
class ContactCMDShell(cmd.Cmd):
    """
    Interactive shell for contactcmd-cli.
    Handles command history and command aliases.
    """
    # This is printed once when the shell starts
    intro = '''
Welcome to ContactCMD! 🚀

This is your personal contact management system. Here are some commands to get started:

  📋 Basic Commands:
    contact add         - Add a new contact (interactive prompts)
    contact list        - List all your contacts
    contact search      - Search for contacts by name or email
    help               - Show all available commands
    
  💡 Quick Examples:
    contact add --name-first John --name-last Doe --email john@example.com
    contact search john
    interaction add     - Record an interaction with a contact
    
  🤖 Natural Language (if enabled):
    nl "add john doe with email john@example.com"
    nl "show recent interactions"
    
Type 'help' for a complete list of commands or 'exit' to quit.
'''

    # This is the prompt displayed before each command input
    prompt = '(contactcmd) '
    
    # Enable tab completion and raw input mode for readline to work
    use_rawinput = True
    completekey = 'tab'
    
    # Command aliases - map natural language commands to snake_case methods
    command_aliases = {
        # Contact commands
        'contact add': 'contact_add',
        'contact list': 'contact_list',
        'contact show': 'contact_show',
        'contact search': 'contact_search',
        'contact missing-email': 'contact_missing_email',
        'contact fix-missing-emails': 'contact_fix_missing_emails',
        'contact missing-names': 'contact_missing_names',
        'contact fix-missing-names': 'contact_fix_missing_names',
        'contact delete': 'contact_delete',
        'contact delete-all': 'contact_delete_all',
        # Interaction commands
        'interaction add': 'interaction_add',
        # Special date commands
        'special-date add': 'special_date_add',
        # Sync commands
        'sync mac': 'sync_mac',
        # Allow for shorter versions too
        'add contact': 'contact_add',
        'list contacts': 'contact_list',
        'search contacts': 'contact_search',
        'show contact': 'contact_show',
        'missing emails': 'contact_missing_email',
        'fix emails': 'contact_fix_missing_emails',
    }
    
    # Helper method for app-wide help
    def do_help(self, arg):
        """
        Display help information about available commands.
        
        Usage:
          help                  - Show list of all available commands
          help <command>        - Show detailed help for a specific command
          help commands         - Show all commands with short descriptions
          help usage            - Show general usage patterns for common tasks
          help options          - Show global options and flags
        """
        if not arg:
            # Call the parent class's help implementation for no args
            super().do_help(arg)
            print("\nFor more detailed help, try:")
            print("  help commands  - List all commands with descriptions")
            print("  help usage     - Show common usage patterns")
            print("  help options   - Show global options and flags")
            return
        
        # Handle help for aliases by converting to snake_case method name
        if arg in self.command_aliases:
            arg = self.command_aliases[arg]
        
        # Check for special help topics
        if arg == 'commands':
            self._show_commands_help()
            return
        elif arg == 'usage':
            self._show_usage_help()
            return
        elif arg == 'options':
            self._show_options_help()
            return
        elif arg.startswith('contact'):
            if arg == 'contact':
                self._show_contact_help()
                return
            elif ' ' in arg:  # Handle subcommands like 'contact list'
                subcommand = arg.split(' ')[1]
                method_name = f"help_contact_{subcommand}"
                if hasattr(self, method_name):
                    getattr(self, method_name)()
                    return
        elif arg.startswith('interaction'):
            if arg == 'interaction':
                self._show_interaction_help()
                return
        elif arg.startswith('special-date'):
            if arg == 'special-date':
                self._show_special_date_help()
                return
        elif arg.startswith('sync'):
            if arg == 'sync':
                self._show_sync_help()
                return
                
        # Otherwise, delegate to parent help for specific commands
        super().do_help(arg)
        
    def _show_contact_help(self):
        """Show help for the contact command and its subcommands"""
        print("\nContact Management Commands:\n")
        print("  contact add         - Add a new contact")
        print("  contact list        - List all contacts with optional sorting")
        print("  contact show        - Show details for a specific contact")
        print("  contact search      - Search for contacts by name or email")
        print("  contact missing-email - List contacts without email addresses")
        print("  contact fix-missing-emails - Interactive UI to add missing emails")
        print("  contact delete      - Delete a contact by ID")
        print("  contact delete-all  - Delete all contacts (use with caution)")
        print("\nFor details on a specific subcommand, type:")
        print("  help contact <subcommand>")
        print("  For example: help contact add")
        
    def _show_interaction_help(self):
        """Show help for the interaction command and its subcommands"""
        print("\nInteraction Commands:\n")
        print("  interaction add     - Add an interaction for a contact")
        print("\nFor details on a specific subcommand, type:")
        print("  help interaction <subcommand>")
        print("  For example: help interaction add")
        
    def _show_special_date_help(self):
        """Show help for the special-date command and its subcommands"""
        print("\nSpecial Date Commands:\n")
        print("  special-date add    - Add a special date for a contact")
        print("\nFor details on a specific subcommand, type:")
        print("  help special-date <subcommand>")
        print("  For example: help special-date add")
        
    def _show_sync_help(self):
        """Show help for the sync command and its subcommands"""
        print("\nSync Commands:\n")
        print("  sync mac            - Sync with macOS Contacts app")
        print("  sync debug          - Check sync compatibility and configuration")
        print("  sync test           - Test sync functionality without accessing APIs")
        print("\nFor details on a specific subcommand, type:")
        print("  help sync <subcommand>")
        print("  For example: help sync mac")
    
    def _show_commands_help(self):
        """Display all commands with their short descriptions"""
        print("\nAvailable Commands:\n")
        
        print("Contact Management:")
        print("  contact add         - Add a new contact")
        print("  contact list        - List all contacts with optional sorting")
        print("  contact show        - Show details for a specific contact")
        print("  contact search      - Search for contacts by name or email")
        print("  contact missing-email - List contacts without email addresses")
        print("  contact fix-missing-emails - Interactive UI to add missing emails")
        print("  contact delete      - Delete a contact by ID")
        
        print("\nInteractions:")
        print("  interaction add     - Add an interaction for a contact")
        
        print("\nSpecial Dates:")
        print("  special-date add    - Add a special date for a contact")
        
        print("\nSync:")
        print("  sync mac            - Sync with macOS Contacts app")
        
        print("\nGeneral:")
        print("  help                - Display this help message")
        print("  quit, exit          - Exit the interactive shell")
        print("  EOF (Ctrl+D)        - Exit the interactive shell")
        
        print("\nNote: The original snake_case commands (e.g., contact_add) also work,"
              "\nbut the more natural kebab-case format shown above is recommended.")
    
    def _show_usage_help(self):
        """Display common usage patterns"""
        print("\nCommon Usage Patterns:\n")
        
        print("Adding a new contact:")
        print("  contact_add --name-first \"John\" --name-last \"Doe\" --email \"john.doe@example.com\"")
        
        print("\nListing contacts:")
        print("  contact_list")
        print("  contact_list --sort-by last_name --order desc")
        print("  contact_list --all")
        
        print("\nSearching contacts:")
        print("  contact_search \"John Doe\"")
        print("  contact_search \"gmail.com\" --case-sensitive")
        
        print("\nManaging missing emails:")
        print("  contact_missing_email")
        print("  contact_fix_missing_emails")
        
        print("\nViewing contact details:")
        print("  contact_show <contact_id>")
        
        print("\nAdding interaction:")
        print("  interaction_add <contact_id> --type \"call\" --notes \"Discussed project timeline\"")
        
        print("\nAdding special date:")
        print("  special_date_add <contact_id> --type \"birthday\" --date \"1990-05-15\" --description \"John's birthday\"")
        
        print("\nSyncing with macOS:")
        print("  sync_mac")
        print("  sync_mac --all --dry-run")
    
    def _show_options_help(self):
        """Display global options and argument patterns"""
        print("\nOption Patterns:\n")
        
        print("Most commands accept options in the format: --option-name value")
        print("Boolean flags are specified as: --flag-name (without a value)")
        
        print("\nCommon Options:")
        print("  --sort-by <field>   - Field to sort by (first_name, last_name, email, created)")
        print("  --order <dir>       - Sort order (asc, desc)")
        print("  --page <num>        - Page number for paginated results (starts at 1)")
        print("  --page-size <num>   - Number of items per page (default 20)")
        print("  --all               - Show all results without pagination")
        print("  --case-sensitive    - Make search case-sensitive")
        
        print("\nSpecial Arguments:")
        print("  <contact_id>        - The UUID of a contact")
        print("  <search_term>       - Text to search for in contacts")
        print("  <YYYY-MM-DD>        - Date format for special dates")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # History file path in the config directory
        config_dir = os.path.join(os.path.expanduser("~"), ".config", "contactcmd")
        hist_file_path = os.path.join(config_dir, "history")
        self._hist_file = hist_file_path
        self._using_readline = False # Assume False initially

        try:
            # Set max history entries
            readline.set_history_length(1000)
            
            # Enable tab completion
            if 'libedit' in readline.__doc__:
                # macOS uses libedit instead of GNU readline
                readline.parse_and_bind("bind ^I rl_complete")
            else:
                # GNU readline
                readline.parse_and_bind("tab: complete")
                
            try:
                # Read history from the file when the shell starts
                readline.read_history_file(hist_file_path)
            except (FileNotFoundError, PermissionError) as e:
                # Ignore if the history file doesn't exist yet or we don't have permission
                pass

            self._using_readline = True
        except (ImportError, AttributeError):
            # readline module is not available (e.g., standard Windows Python)
            self._history = [] # Fallback: store history in a list (no file persistence or editing)
            self._hist_file = None # No history file to save/load

    def precmd(self, line):
        """
        Called just before a command is dispatched.
        This method handles command aliases for a more natural CLI experience.
        """
        if not line.strip():
            return line
            
        # Check if the input matches any of our aliases
        for alias, cmd in self.command_aliases.items():
            # Check if the line starts with this alias
            if line.startswith(alias):
                # Replace only at the beginning to preserve arguments
                args = line[len(alias):].strip()
                new_line = f"{cmd} {args}" if args else cmd
                return new_line
                
        # If no alias matched, return the line unchanged
        return line
        
    def completenames(self, text, *ignored):
        """
        Tab completion for commands.
        This method is called when tab is pressed to complete a command name.
        """
        # Get the default command completion from cmd.Cmd
        commands = super().completenames(text, *ignored)
        
        # Add the natural language command aliases that match
        for alias in self.command_aliases:
            if alias.startswith(text):
                commands.append(alias)
                
        # If text contains a space, it might be a multi-word command
        if ' ' in text:
            # Get the main command and partial subcommand
            parts = text.split(' ', 1)
            main_cmd = parts[0]
            partial_subcmd = parts[1] if len(parts) > 1 else ''
            
            # Complete subcommands for specific main commands
            if main_cmd == 'contact':
                subcommands = ['add', 'list', 'show', 'search', 'missing-email', 'fix-missing-emails', 
                               'missing-names', 'fix-missing-names', 'delete', 'delete-all']
                for subcmd in subcommands:
                    if (not partial_subcmd or subcmd.startswith(partial_subcmd)) and f"{main_cmd} {subcmd}".startswith(text):
                        commands.append(f"{main_cmd} {subcmd}")
            elif main_cmd == 'interaction':
                subcommands = ['add']
                for subcmd in subcommands:
                    if (not partial_subcmd or subcmd.startswith(partial_subcmd)) and f"{main_cmd} {subcmd}".startswith(text):
                        commands.append(f"{main_cmd} {subcmd}")
            elif main_cmd == 'special-date':
                subcommands = ['add']
                for subcmd in subcommands:
                    if (not partial_subcmd or subcmd.startswith(partial_subcmd)) and f"{main_cmd} {subcmd}".startswith(text):
                        commands.append(f"{main_cmd} {subcmd}")
            elif main_cmd == 'sync':
                subcommands = ['mac', 'debug', 'test']
                for subcmd in subcommands:
                    if (not partial_subcmd or subcmd.startswith(partial_subcmd)) and f"{main_cmd} {subcmd}".startswith(text):
                        commands.append(f"{main_cmd} {subcmd}")
        
        return commands
        
    def complete(self, text, state):
        """
        Return the next possible completion for 'text'.
        This is called by the readline library to provide tab completion.
        """
        # Get the current line and cursor position
        origline = readline.get_line_buffer()
        line = origline.lstrip()
        stripped = len(origline) - len(line)
        begidx = readline.get_begidx() - stripped
        endidx = readline.get_endidx() - stripped
        
        # If we're at the beginning of the line, complete command names and aliases
        if begidx == 0:
            commands = []
            # Add command names from do_* methods
            for cmd in self.get_names():
                if cmd.startswith('do_'):
                    commands.append(cmd[3:])
            
            # Add command aliases
            for alias in self.command_aliases:
                commands.append(alias)
            
            # Filter to ones that match the current text
            matches = [c for c in commands if c.startswith(text)]
            
            try:
                return matches[state]
            except IndexError:
                return None
        
        # Handle main command completions (like "contact" + space + TAB)
        parts = shlex.split(line[:begidx] + text) if line[:begidx].strip() else []
        
        # Special case: complete main command options
        if len(parts) == 1 or (len(parts) == 0 and text.strip()):
            main_cmd = parts[0] if parts else text.strip()
            
            # Handle top-level commands like "contact", "interaction", etc.
            if main_cmd in ['contact', 'interaction', 'special-date', 'sync']:
                if main_cmd == 'contact':
                    options = ['add', 'list', 'show', 'search', 'missing-email', 'fix-missing-emails', 'delete', 'delete-all']
                elif main_cmd == 'interaction':
                    options = ['add']
                elif main_cmd == 'special-date':
                    options = ['add']
                elif main_cmd == 'sync':
                    options = ['mac', 'debug', 'test']
                
                # Filter options by current text if any
                if text:
                    options = [opt for opt in options if opt.startswith(text)]
                
                try:
                    return options[state]
                except IndexError:
                    return None
        
        # Handle two-word command completions (like "contact fix-" + TAB)
        if len(parts) >= 1:
            cmd = parts[0]
            
            # Special case for "contact" command
            if cmd == 'contact' and len(text) > 0:
                subcommands = ['add', 'list', 'show', 'search', 'missing-email', 'fix-missing-emails', 
                              'missing-names', 'fix-missing-names', 'delete', 'delete-all']
                matches = [subcmd for subcmd in subcommands if subcmd.startswith(text)]
                try:
                    return matches[state]
                except IndexError:
                    return None
            
            # Special case for "interaction" command
            elif cmd == 'interaction' and len(text) > 0:
                subcommands = ['add']
                matches = [subcmd for subcmd in subcommands if subcmd.startswith(text)]
                try:
                    return matches[state]
                except IndexError:
                    return None
            
            # Special case for "special-date" command
            elif cmd == 'special-date' and len(text) > 0:
                subcommands = ['add']
                matches = [subcmd for subcmd in subcommands if subcmd.startswith(text)]
                try:
                    return matches[state]
                except IndexError:
                    return None
            
            # Special case for "sync" command
            elif cmd == 'sync' and len(text) > 0:
                subcommands = ['mac', 'debug', 'test']
                matches = [subcmd for subcmd in subcommands if subcmd.startswith(text)]
                try:
                    return matches[state]
                except IndexError:
                    return None
                    
            # Convert command alias to method name if it exists
            if cmd in self.command_aliases:
                cmd = self.command_aliases[cmd]
            
            # Try to find specific completer for this command
            try:
                # Check if there's a completer method for this command
                compfunc = getattr(self, f'complete_{cmd}')
                return compfunc(text, line, begidx, endidx, state)
            except AttributeError:
                # No specific completer, use completedefault
                pass
        
        # Fall back to default completer
        args = self.completedefault(text, line, begidx, endidx)
        try:
            return args[state]
        except (IndexError, TypeError):
            return None
    
    def completedefault(self, text, line, begidx, endidx):
        """
        Default tab completion for arguments.
        This method is called when tab is pressed to complete arguments.
        """
        # Extract the command part to determine what we're completing
        parts = shlex.split(line[:begidx]) if line[:begidx].strip() else []
        remaining = line[begidx:endidx] + text
        
        # Get the command (first word on the line)
        command = parts[0] if parts else ""
        
        # Convert command alias to method name if it exists
        if command in self.command_aliases:
            command = self.command_aliases[command]
        
        # Handle subcommands (like 'contact list', 'contact add', etc.)
        if len(parts) > 1 and parts[0] == 'contact':
            subcommand = parts[1]
            full_command = f"{parts[0]} {subcommand}"
            
            # Check for known subcommands
            if subcommand == 'list' or full_command == 'contact list':
                options = ['--sort-by', '--order', '--page', '--page-size', '--all']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
                
            elif subcommand == 'add' or full_command == 'contact add':
                options = ['--name-first', '--name-last', '--email']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
                
            elif subcommand == 'search' or full_command == 'contact search':
                options = ['--case-sensitive', '--sort-by', '--order', '--page', '--page-size', '--all']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
                
            # Complete the subcommand itself with available contact operations
            elif parts[0] == 'contact' and not subcommand.startswith('-'):
                options = ['add', 'list', 'show', 'search', 'missing-email', 'fix-missing-emails', 'delete', 'delete-all']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
        
        # Handle 'interaction' command and subcommands
        elif len(parts) > 1 and parts[0] == 'interaction':
            subcommand = parts[1]
            
            if subcommand == 'add' or f"{parts[0]} {subcommand}" == 'interaction add':
                options = ['--type', '--notes']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
                
            # Complete the subcommand itself with available interaction operations
            elif parts[0] == 'interaction' and not subcommand.startswith('-'):
                options = ['add']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
                
        # Handle 'special-date' command and subcommands
        elif len(parts) > 1 and (parts[0] == 'special-date' or parts[0] == 'special_date'):
            subcommand = parts[1]
            
            if subcommand == 'add' or f"{parts[0]} {subcommand}" == 'special-date add':
                options = ['--type', '--date', '--description']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
                
            # Complete the subcommand itself with available special-date operations
            elif (parts[0] == 'special-date' or parts[0] == 'special_date') and not subcommand.startswith('-'):
                options = ['add']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
                
        # Handle 'sync' command and subcommands
        elif len(parts) > 1 and parts[0] == 'sync':
            subcommand = parts[1]
            
            if subcommand == 'mac' or f"{parts[0]} {subcommand}" == 'sync mac':
                options = ['--all', '--dry-run', '--debug']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
                
            # Complete the subcommand itself with available sync operations
            elif parts[0] == 'sync' and not subcommand.startswith('-'):
                options = ['mac', 'debug', 'test']
                if text:
                    return [opt for opt in options if opt.startswith(text)]
                return options
                
        # Single-word command completions
        elif command in ['contact_list']:
            options = ['--sort-by', '--order', '--page', '--page-size', '--all']
            if text:
                return [opt for opt in options if opt.startswith(text)]
            return options
            
        elif command in ['contact_search']:
            options = ['--case-sensitive', '--sort-by', '--order', '--page', '--page-size', '--all']
            if text:
                return [opt for opt in options if opt.startswith(text)]
            return options
            
        elif command in ['interaction_add']:
            options = ['--type', '--notes']
            if text:
                return [opt for opt in options if opt.startswith(text)]
            return options
            
        elif command in ['special_date_add']:
            options = ['--type', '--date', '--description']
            if text:
                return [opt for opt in options if opt.startswith(text)]
            return options
            
        elif command in ['sync_mac']:
            options = ['--all', '--dry-run', '--debug']
            if text:
                return [opt for opt in options if opt.startswith(text)]
            return options
            
        # For completing specific option values
        for part in parts:
            if part == '--sort-by' and text == '':
                return ['first_name', 'last_name', 'email', 'created']
            elif part == '--order' and text == '':
                return ['asc', 'desc']
            elif part == '--type' and 'interaction' in line and text == '':
                return ['call', 'meeting', 'email', 'note']
            elif part == '--type' and 'special' in line and text == '':
                return ['birthday', 'anniversary', 'reminder']
                
        # Default: no completion suggestions
        return []

    def postcmd(self, stop, line):
        # Called after a command is executed
        return stop # Return the stop flag unchanged
        
    def default(self, line):
        """Handle unknown commands"""
        # Check if this is a help request with ? or --help
        if line.endswith(" ?") or " ? " in line:
            cmd = line.split(" ?")[0].strip()
            return self.do_help(cmd)
        elif line.endswith(" --help") or " --help " in line:
            cmd = line.split(" --help")[0].strip()
            return self.do_help(cmd)
        
        # Try natural language processing
        from llm import init_llm_on_first_use, ensure_llm_model
        from nl_handler import NLCommandHandler
        from llm.manager import get_config
        
        config = get_config()
        
        # If enable_llm is not set, prompt for first-time setup
        if "enable_llm" not in config:
            model_path = init_llm_on_first_use()
        else:
            model_path = ensure_llm_model()
        
        if not model_path:
            # LLM is disabled, show original error
            print(f"*** Unknown syntax: {line}")
            print(f"Note: Natural language processing is disabled. To enable, run: llm enable")
            return False
        
        # Process with natural language
        try:
            print(f"\n🤖 Processing natural language command...")
            handler = NLCommandHandler(model_path)
            result = handler.process_command(line)
            
            if result:
                print(f"\n💡 Suggested command: {result}")
                
                # Ask if user wants to execute the command
                try:
                    response = input("\nExecute this command? [Y/n]: ").strip().lower()
                    if response in ['', 'y', 'yes']:
                        # Execute the command in the shell
                        return self.onecmd(result)
                    else:
                        print("Command not executed.")
                except (EOFError, KeyboardInterrupt):
                    print("\nCommand not executed.")
            else:
                print("Could not understand the command. Please try rephrasing.")
        except Exception as e:
            print(f"Error processing natural language: {e}")
            print(f"*** Unknown syntax: {line}")
        
        return False  # Don't exit the shell

    def do_quit(self, arg):
        """Quit the interactive shell."""
        print("Exiting contactcmd shell.")
        # Save history if using readline and the history file path is set
        if self._using_readline and self._hist_file:
            try:
                import readline
                # Write current history to file
                readline.write_history_file(self._hist_file)
            except Exception:
                # Fail silently for history saving errors on exit
                pass
        return True # Returning True stops the cmdloop

    def do_exit(self, arg):
        """Exit the interactive shell (alias for quit)."""
        return self.do_quit(arg)

    def do_EOF(self, arg):
        """Exit on Ctrl+D."""
        print() # Print a newline after ^D input
        return self.do_quit(arg)
        
    # Helper for Manual Argument Parsing
    def _parse_shell_args(self, arg_string):
        """
        Parses a shell-like argument string into a dictionary of keyword arguments.
        Handles --option value and --flag (boolean).
        Assumes arguments starting with '--' are options.
        Prints a warning for positional arguments.
        """
        kwargs = {}
        # Safely split the argument string, respecting quotes
        args = shlex.split(arg_string)
        i = 0
        while i < len(args):
            arg = args[i]
            if arg.startswith('--'):
                # It's an option. Get the option name (without -- and with underscores)
                option_name = arg[2:].replace('-', '_')
                i += 1
                # Check if the next argument exists and is NOT another option
                if i < len(args) and not args[i].startswith('--'):
                    # The next argument is the value for this option
                    kwargs[option_name] = args[i]
                    i += 1
                else:
                    # No value provided, assume it's a boolean flag set to True
                    kwargs[option_name] = True
            else:
                # This argument does not start with --. It's a positional argument.
                # Our current do_* methods are designed to handle options or
                # a specific leading positional argument (like contact ID) separately.
                # This general parser helper will warn about unexpected positionals here.
                print(f"Warning: Positional argument '{arg}' found. This parser only handles --options.")
                # We still need to consume the argument to move on
                i += 1

        return kwargs
        
    def do_contact_add(self, arg):
        """
        Add a new contact to the database.
        
        USAGE:
          contact_add --name-first <first_name> --name-last <last_name> --email <email>
          
        REQUIRED OPTIONS:
          --name-first <text>   - First name of the contact
          --name-last <text>    - Last name of the contact
          --email <text>        - Email address of the contact
          
        EXAMPLES:
          contact_add --name-first "John" --name-last "Doe" --email "john.doe@example.com"
          contact_add --name-first "Jane" --name-last "Smith" --email "jane.smith@company.com"
          
        NOTES:
          - All fields are required to create a valid contact
          - The command will return the UUID of the new contact
          - You'll need this UUID to add interactions or special dates
        """
        # Parse the arguments using the helper
        kwargs = self._parse_shell_args(arg)

        # Validate required arguments based on your function signature
        required_args = ['name_first', 'name_last', 'email']
        missing = [arg for arg in required_args if arg not in kwargs]
        if missing:
            print(f"Error: Missing required options: {', '.join(['--' + m.replace('_', '-') for m in missing])}", file=sys.stderr)
            self.help_contact_add()  # Show help for correct usage
            return  # Stop processing this command

        # Call the actual underlying add_contact function
        try:
            # Import the function here to avoid circular imports
            from contact import add_contact
            # Pass the parsed keyword arguments directly to your function
            added_contact = add_contact(**kwargs)
            print(f"Contact '{added_contact.name_first} {added_contact.name_last}' added successfully.")
            print(f"Contact ID: {added_contact.local_id}")
        except Exception as e:
            # Catch any exceptions raised by your add_contact function (e.g., DB errors, validation)
            print(f"Error adding contact: {e}", file=sys.stderr)
            # Optional: print traceback for detailed debugging
            # traceback.print_exc(file=sys.stderr)

    def help_contact_add(self):
        """Prints help for contact_add."""
        # Use the docstring of the do_* method as the help text
        print(self.do_contact_add.__doc__)
        
    # Helper for Parsing Pagination/Sorting Options
    def _parse_pagination_args(self, kwargs):
        """
        Parses and validates pagination/sorting arguments from a kwargs dictionary.
        Returns a dictionary of cleaned keyword arguments or raises ValueError.
        """
        converted_kwargs = {}
        
        # --sort-by
        if 'sort_by' in kwargs:
            converted_kwargs['sort_by'] = kwargs['sort_by']
            # Optional: Add validation here to check if sort_by field is valid
            
        # --order
        if 'order' in kwargs:
            order_val = kwargs['order'].lower()
            if order_val not in ['asc', 'desc']:
                raise ValueError(f"Invalid value for --order: '{kwargs['order']}'. Must be 'asc' or 'desc'.")
            converted_kwargs['order'] = order_val
            
        # --page
        if 'page' in kwargs:
            try:
                page_num = int(kwargs['page'])
                if page_num < 1:
                    raise ValueError("Page number must be 1 or greater.")
                converted_kwargs['page'] = page_num
            except ValueError as e:
                raise ValueError(f"Invalid value for --page: {e}")
                
        # --page-size
        MAX_PAGE_SIZE = 100  # Based on the function signature
        if 'page_size' in kwargs:
            try:
                page_size_val = int(kwargs['page_size'])
                if page_size_val < 1 or page_size_val > MAX_PAGE_SIZE:
                    raise ValueError(f"Page size must be between 1 and {MAX_PAGE_SIZE}.")
                converted_kwargs['page_size'] = page_size_val
            except ValueError as e:
                raise ValueError(f"Invalid value for --page-size: {e}")
                
        # --all (boolean flag)
        converted_kwargs['all_pages'] = kwargs.get('all', False)
        
        return converted_kwargs
        
    def do_contact_list(self, arg):
        """
        List contacts with optional sorting and pagination.
        
        USAGE:
          contact_list [options]
          
        OPTIONS:
          --sort-by <field>     - Field to sort by: first_name, last_name, email, created
          --order <direction>   - Sort order: asc (ascending) or desc (descending)
          --page <number>       - Page number (starting from 1)
          --page-size <number>  - Number of contacts per page (1-100, default 20)
          --all                 - Show all results without pagination
          
        EXAMPLES:
          contact_list                                 - List contacts with default sorting
          contact_list --sort-by last_name             - Sort by last name (ascending)
          contact_list --sort-by email --order desc    - Sort by email (descending)
          contact_list --page 2 --page-size 10         - Show second page, 10 items per page
          contact_list --all                           - Show all contacts without pagination
          
        NOTES:
          - Default sort is by creation date (newest first)
          - Default page size is 20 contacts
        """
        # Parse the arguments using the helper
        try:
            kwargs = self._parse_shell_args(arg)
        except Exception as e:
            print(f"Error parsing arguments: {e}", file=sys.stderr)
            self.help_contact_list()
            return
        
        try:
            converted_kwargs = self._parse_pagination_args(kwargs)
        except ValueError as e:
            print(f"Error parsing pagination arguments: {e}", file=sys.stderr)
            self.help_contact_list()  # Show help for argument parsing errors
            return
        
        # Call the actual underlying list_contacts function
        try:
            print("Fetching contacts...")
            
            # Import the function here to avoid circular imports
            from contact import list_contacts
            
            # Pass the validated and type-converted arguments
            contacts = list_contacts(**converted_kwargs)
            
        except ImportError as e:
            print(f"Error importing contact module: {e}", file=sys.stderr)
        except ValueError as e:
            print(f"Invalid value in list parameters: {e}", file=sys.stderr)
        except Exception as e:
            # Catch any other exceptions raised by list_contacts function
            print(f"Error listing contacts: {e}", file=sys.stderr)
            # Uncomment for debugging: traceback.print_exc(file=sys.stderr)
    
    def help_contact_list(self):
        """Prints help for contact_list."""
        print(self.do_contact_list.__doc__)
        
    def do_contact_show(self, arg):
        """
        Show detailed information about a specific contact.
        
        USAGE:
          contact_show <contact_id>
          
        ARGUMENTS:
          <contact_id>          - The UUID of the contact to display
          
        EXAMPLES:
          contact_show 12345678-1234-5678-1234-567812345678
          
        NOTES:
          - Displays all contact information including:
            * Basic info (name, email)
            * Associated interactions
            * Special dates
            * Sync status and timestamps
          - You can find contact IDs by using contact_list or contact_search
        """
        # Split the argument string
        parts = shlex.split(arg)
        
        # Extract the required positional argument (the contact ID)
        if not parts or parts[0].startswith('--'):
            print("Error: Contact ID is required.", file=sys.stderr)
            self.help_contact_show()
            return
        
        contact_id_str = parts[0]
        
        # Validate the contact ID format (optional)
        try:
            # Simple format validation to catch obvious errors
            # For UUID validation, would do: uuid.UUID(contact_id_str)
            # But we'll keep it simple and just ensure it's not empty
            if not contact_id_str.strip():
                raise ValueError("Contact ID cannot be empty")
            
            contact_id_for_func = contact_id_str
        except ValueError as e:
            print(f"Error: Invalid contact ID format: {e}", file=sys.stderr)
            self.help_contact_show()
            return
        
        # Call the show_contact function
        try:
            print(f"Showing details for contact ID: {contact_id_for_func}")
            
            # Import the function here to avoid circular imports
            from contact import show_contact
            
            # Call the function with the contact ID - note that this displays the contact
            # directly rather than returning the contact object
            show_contact(contact_id_for_func)
                
        except Exception as e:
            print(f"Error showing contact: {e}", file=sys.stderr)
            # For debugging: traceback.print_exc(file=sys.stderr)
    
    def help_contact_show(self):
        """Prints help for contact_show."""
        print(self.do_contact_show.__doc__)
        
    def do_interaction_add(self, arg):
        """
        Add a new interaction record for a contact.
        
        USAGE:
          interaction_add <contact_id> --type <interaction_type> --notes <text>
          
        ARGUMENTS:
          <contact_id>          - The UUID of the contact
          
        REQUIRED OPTIONS:
          --type <text>         - Type of interaction (e.g., call, email, meeting, note)
          --notes <text>        - Content/description of the interaction
          
        EXAMPLES:
          interaction_add 12345678-1234-5678-1234-567812345678 --type call --notes "Discussed project timeline"
          interaction_add 12345678-1234-5678-1234-567812345678 --type meeting --notes "Initial client consultation"
          
        NOTES:
          - Interactions are automatically timestamped
          - Common interaction types:
            * call - Phone conversation
            * email - Email communication
            * meeting - In-person or virtual meeting
            * note - General notes or follow-ups
          - You can use your own custom interaction types
        """
        # Safely split the argument string
        try:
            parts = shlex.split(arg)
        except Exception as e:
            print(f"Error parsing arguments: {e}", file=sys.stderr)
            self.help_interaction_add()
            return
            
        # Extract the required positional argument (the contact ID)
        if not parts or parts[0].startswith('--'):
            print("Error: Contact ID is required.", file=sys.stderr)
            self.help_interaction_add()
            return
        
        contact_id_str = parts[0]
        
        # Validate the contact ID (Optional but good practice)
        try:
            # If you have a validation function, call it here
            # Or directly use uuid.UUID() if validating format
            import uuid
            uuid.UUID(contact_id_str)  # Just format validation
            contact_id_for_func = contact_id_str
            
        except ValueError as e:
            print(f"Error: Invalid contact ID format: {e}", file=sys.stderr)
            self.help_interaction_add()
            return
        
        # Parse the rest of the arguments as options
        try:
            remaining_args_string = shlex.join(parts[1:])  # Join the rest back into a string
            kwargs = self._parse_shell_args(remaining_args_string)
        except Exception as e:
            print(f"Error parsing options: {e}", file=sys.stderr)
            self.help_interaction_add()
            return
        
        # Validate required options (--type, --notes)
        required_options = ['type', 'notes']
        missing_options = [opt for opt in required_options if opt not in kwargs]
        if missing_options:
            print(f"Error: Missing required options: {', '.join(['--' + m.replace('_', '-') for m in missing_options])}", file=sys.stderr)
            self.help_interaction_add()
            return
        
        # Call the add_interaction function
        try:
            print(f"Adding interaction for contact ID: {contact_id_for_func}")
            
            # Import the function here to avoid circular imports
            from interaction import add_interaction
            
            # Call the function with the contact ID and parsed options
            add_interaction(contact_id=contact_id_for_func, type=kwargs['type'], text=kwargs['notes'])
            
            print("Interaction added successfully.")
            
        except ImportError as e:
            print(f"Error importing interaction module: {e}", file=sys.stderr)
        except ValueError as e:
            print(f"Invalid value: {e}", file=sys.stderr)
        except Exception as e:
            # Catch any other exceptions raised by the add_interaction function
            print(f"Error adding interaction: {e}", file=sys.stderr)
            # Uncomment for debugging: traceback.print_exc(file=sys.stderr)
    
    def help_interaction_add(self):
        """Prints help for interaction_add."""
        print(self.do_interaction_add.__doc__)
        
    def do_special_date_add(self, arg):
        """
        Add a special date (birthday, anniversary, etc.) for a contact.
        
        USAGE:
          special_date_add <contact_id> --type <date_type> --date <YYYY-MM-DD> [--description <text>]
          
        ARGUMENTS:
          <contact_id>          - The UUID of the contact
          
        REQUIRED OPTIONS:
          --type <text>         - Type of special date (e.g., birthday, anniversary)
          --date <YYYY-MM-DD>   - The date in ISO format (year-month-day)
          
        OPTIONAL OPTIONS:
          --description <text>  - Additional details about the special date
          
        EXAMPLES:
          special_date_add 12345678-1234-5678-1234-567812345678 --type birthday --date 1990-05-15
          special_date_add 12345678-1234-5678-1234-567812345678 --type anniversary --date 2020-06-12 --description "Work anniversary"
          
        NOTES:
          - Date must be in YYYY-MM-DD format (e.g., 1990-05-15)
          - Common special date types:
            * birthday - Person's date of birth
            * anniversary - Work, relationship, or other milestone anniversary
            * reminder - Any other date to remember
          - You can use your own custom date types
          - The description field is optional but recommended for clarity
        """
        # Safely split the argument string
        try:
            parts = shlex.split(arg)
        except Exception as e:
            print(f"Error parsing arguments: {e}", file=sys.stderr)
            self.help_special_date_add()
            return
        
        # Extract the required positional argument (the contact ID)
        if not parts or parts[0].startswith('--'):
            print("Error: Contact ID is required.", file=sys.stderr)
            self.help_special_date_add()
            return
        
        contact_id_str = parts[0]
        
        # Validate the contact ID format
        try:
            # Validate format using UUID
            import uuid
            uuid.UUID(contact_id_str)  # Just format validation
            contact_id_for_func = contact_id_str
            
        except ValueError as e:
            print(f"Error: Invalid contact ID format: {e}", file=sys.stderr)
            self.help_special_date_add()
            return
        
        # Parse the rest of the arguments as options
        try:
            remaining_args_string = shlex.join(parts[1:])  # Join the rest back into a string
            kwargs = self._parse_shell_args(remaining_args_string)
        except Exception as e:
            print(f"Error parsing options: {e}", file=sys.stderr)
            self.help_special_date_add()
            return
        
        # Validate required options (--type, --date)
        required_options = ['type', 'date']
        missing_options = [opt for opt in required_options if opt not in kwargs]
        if missing_options:
            print(f"Error: Missing required options: {', '.join(['--' + m.replace('_', '-') for m in missing_options])}", file=sys.stderr)
            self.help_special_date_add()
            return
        
        # Validate date format (YYYY-MM-DD)
        date_str = kwargs.get('date')
        try:
            # Attempt to parse the date string into a datetime object
            import datetime
            date_obj = datetime.datetime.strptime(date_str, '%Y-%m-%d')
            # Keep as string as per the function signature
            date_for_func = date_str
        except ValueError:
            print(f"Error: Invalid date format for '{date_str}'. Must be YYYY-MM-DD.", file=sys.stderr)
            self.help_special_date_add()
            return
        
        # Call the add_special_date function
        try:
            print(f"Adding special date for contact ID: {contact_id_for_func}")
            
            # Import the function here to avoid circular imports
            from special_date import add_special_date
            
            # Call the function with the contact ID and parsed options
            add_special_date(
                contact_id=contact_id_for_func,
                type=kwargs['type'],
                date=date_for_func,
                description=kwargs.get('description', '')  # Optional, provide default if missing
            )
            
            print("Special date added successfully.")
            
        except ImportError as e:
            print(f"Error importing special_date module: {e}", file=sys.stderr)
        except ValueError as e:
            print(f"Invalid value provided: {e}", file=sys.stderr)
        except Exception as e:
            # Catch any other exceptions raised by the add_special_date function
            print(f"Error adding special date: {e}", file=sys.stderr)
            # Uncomment for debugging: traceback.print_exc(file=sys.stderr)
    
    def help_special_date_add(self):
        """Prints help for special_date_add."""
        print(self.do_special_date_add.__doc__)
        
    def do_sync_mac(self, arg):
        """
        Sync contacts from macOS Contacts app to ContactCMD (macOS only).
        
        USAGE:
          sync_mac [options]
          
        OPTIONS:
          --all                 - Sync all contacts, not just recent changes
          --dry-run             - Show what would happen without making changes
          --debug               - Display detailed debug information during sync
          
        EXAMPLES:
          sync_mac              - Standard sync (only changes since last sync)
          sync_mac --all        - Force sync of all contacts
          sync_mac --dry-run    - Preview sync operations without applying them
          sync_mac --all --dry-run --debug  - Full preview with detailed output
          
        NOTES:
          - First-time use will request permission to access your Mac Contacts
          - One-way sync only (Mac → ContactCMD, not ContactCMD → Mac)
          - Without --all, only contacts changed since the last sync are processed
          - Basic contact info is synced: names, emails, and birthdays
          - Only available on macOS platforms
        """
        # Parse arguments (only flags expected)
        kwargs = self._parse_shell_args(arg)
        
        # Manually process boolean flags and set defaults to False
        sync_kwargs = {
            'all_contacts': kwargs.get('all', False),  # Map 'all' from parser to 'all_contacts' for function
            'dry_run': kwargs.get('dry_run', False),
            'debug': kwargs.get('debug', False),
        }
        
        # Call the actual underlying sync_mac_contacts function
        try:
            print("Starting macOS sync...")  # Indicate action
            
            # Import here to avoid importing Mac-specific modules on non-Mac platforms
            import platform
            if platform.system() == 'Darwin':
                from sync.mac import sync_mac_contacts
                sync_mac_contacts(**sync_kwargs)
                print("macOS sync process completed.")  # Adjust message based on function's behavior
            else:
                print("Error: macOS contacts sync is only available on macOS")
                
        except Exception as e:
            # Catch any exceptions raised by your sync_mac_contacts function
            print(f"Error during macOS sync: {e}", file=sys.stderr)
            # traceback.print_exc(file=sys.stderr)
    
    def help_sync_mac(self):
        """Prints help for sync_mac."""
        print(self.do_sync_mac.__doc__)
        
    def do_contact_search(self, arg):
        """
        Search for contacts by name or email.
        
        USAGE:
          contact_search <search_term> [options]
          
        ARGUMENTS:
          <search_term>         - Text to search for (can be quoted)
          
        OPTIONS:
          --case-sensitive      - Make search case-sensitive (default: not case-sensitive)
          --sort-by <field>     - Field to sort by: first_name, last_name, email, created
          --order <direction>   - Sort order: asc (ascending) or desc (descending)
          --page <number>       - Page number (starting from 1)
          --page-size <number>  - Number of contacts per page (1-100, default 20)
          --all                 - Show all results without pagination
          
        EXAMPLES:
          contact_search "John Doe"                    - Find contacts matching "John Doe"
          contact_search "gmail.com" --case-sensitive  - Find contacts with exactly "gmail.com"
          contact_search smith --sort-by last_name     - Find "smith", sort by last name
          contact_search "doe" --all                   - Find all "doe" without pagination
          
        NOTES:
          - Search looks through first name, last name, and email fields
          - Search is case-insensitive by default
          - For exact matches, use --case-sensitive option
          - Results are paginated by default (20 items per page)
          - Use --all to display all results without pagination
          - To find contacts without email addresses, use contact_missing_email command
        """
        # If arg is empty, show help and return
        if not arg.strip():
            self.help_contact_search()
            return
            
        # Split the argument string safely
        try:
            parts = shlex.split(arg)
        except Exception as e:
            print(f"Error parsing arguments: {e}", file=sys.stderr)
            self.help_contact_search()
            return
            
        # Extract the required positional argument (the search term)
        if not parts or parts[0].startswith('--'):
            print("Error: Search term is required.", file=sys.stderr)
            self.help_contact_search()
            return
        
        search_term = parts[0]
        
        # Parse the rest of the arguments as options
        remaining_args_string = shlex.join(parts[1:])  # Join the rest back into a string
        kwargs = self._parse_shell_args(remaining_args_string)
        
        # Manually handle specific options for search
        search_kwargs = {
            'query': search_term,  # Pass the positional search term as 'query' keyword argument
        }
        
        # --case-sensitive (boolean flag)
        search_kwargs['case_sensitive'] = kwargs.get('case_sensitive', False)
        
        # Parse pagination and sorting options (reuse helper)
        try:
            pagination_kwargs = self._parse_pagination_args(kwargs)
            # Map all_pages to all_results as expected by the search_contacts function
            if 'all_pages' in pagination_kwargs:
                pagination_kwargs['all_results'] = pagination_kwargs.pop('all_pages')
            # Merge pagination args into search_kwargs
            search_kwargs.update(pagination_kwargs)
            
        except ValueError as e:
            print(f"Error parsing arguments: {e}", file=sys.stderr)
            self.help_contact_search()  # Show help for argument parsing errors
            return
        
        # Call the actual underlying search_contacts function
        try:
            print(f"Searching for '{search_term}'...")  # Indicate action
            
            # Import the function here to avoid circular imports
            from contact import search_contacts
            
            # Call the function with the query and options
            contacts = search_contacts(**search_kwargs)
            
        except Exception as e:
            # Catch any exceptions raised by the search_contacts function
            print(f"Error searching contacts: {e}", file=sys.stderr)
            # traceback.print_exc(file=sys.stderr)
    
    def help_contact_search(self):
        """Prints help for contact_search."""
        print(self.do_contact_search.__doc__)
        
    def do_contact_delete(self, arg):
        """
        Delete a contact from the database.
        
        USAGE:
          contact_delete <contact_id>
          
        ARGUMENTS:
          <contact_id>          - The UUID of the contact to delete
          
        EXAMPLES:
          contact_delete 12345678-1234-5678-1234-567812345678
          
        NOTES:
          - You will be asked to confirm deletion by typing 'DELETE' in uppercase
          - This action is permanent and cannot be undone
          - All related data (interactions, special dates) will also be deleted
          - Deleting a contact also removes it from sync operations
        """
        # Safely split the argument string
        try:
            parts = shlex.split(arg)
        except Exception as e:
            print(f"Error parsing arguments: {e}", file=sys.stderr)
            self.help_contact_delete()
            return
        
        # Check if exactly one argument (the ID) is provided
        if not parts or parts[0].startswith('--'):
            print("Error: Contact ID is required.", file=sys.stderr)
            self.help_contact_delete()
            return
        
        contact_id_str = parts[0]
        
        # Validate the contact ID format
        try:
            # Validate format using UUID
            import uuid
            uuid.UUID(contact_id_str)  # Just format validation
            contact_id_for_func = contact_id_str
            
        except ValueError as e:
            print(f"Error: Invalid contact ID format: {e}", file=sys.stderr)
            self.help_contact_delete()
            return
        
        # Call the delete_contact function
        try:
            print(f"Attempting to delete contact ID: {contact_id_for_func}")
            
            # Import the function here to avoid circular imports
            from contact import delete_contact
            
            # Call the function with the contact ID
            result = delete_contact(contact_id_for_func)
            
            # No need to print success message here as delete_contact already handles that
            
        except ImportError as e:
            print(f"Error importing contact module: {e}", file=sys.stderr)
        except ValueError as e:
            print(f"Invalid contact ID: {e}", file=sys.stderr)
        except Exception as e:
            # Catch any other exceptions raised by delete_contact
            print(f"Error deleting contact: {e}", file=sys.stderr)
            # Uncomment for debugging: traceback.print_exc(file=sys.stderr)
    
    def help_contact_delete(self):
        """Prints help for contact_delete."""
        print(self.do_contact_delete.__doc__)
        
    def do_contact_missing_email(self, arg):
        """
        List contacts without email addresses.
        
        USAGE:
          contact_missing_email
          
        DESCRIPTION:
          This command displays all contacts in the database that don't have
          an email address. The contacts are shown in a table format with their
          ID, first name, and last name.
          
        NOTES:
          - If all contacts have email addresses, a success message is shown
          - Use contact_fix_missing_emails to interactively add emails
          
        EXAMPLES:
          contact_missing_email
        """
        try:
            # Import the function here to avoid circular imports
            from contact.missing_email import list_contacts_without_email
            
            # Call function (no arguments required)
            list_contacts_without_email()
            
        except ImportError as e:
            print(f"Error importing missing_email module: {e}", file=sys.stderr)
        except Exception as e:
            print(f"Error listing contacts without email: {e}", file=sys.stderr)
            # Uncomment for debugging: traceback.print_exc(file=sys.stderr)
    
    def help_contact_missing_email(self):
        """Prints help for contact_missing_email."""
        print(self.do_contact_missing_email.__doc__)
        
    def do_contact_fix_missing_emails(self, arg):
        """
        Interactive session to add email addresses to contacts missing them.
        
        USAGE:
          contact_fix_missing_emails
          
        DESCRIPTION:
          This command starts an interactive session to manage contacts without
          email addresses. For each contact, you'll be presented with options to:
          
          a) Add an email address
          s) Skip to the next contact
          d) Discard/exit the session
          
        The command will track how many contacts you've processed, updated,
        and skipped throughout the session.
        
        NOTES:
          - First run contact_missing_email to see which contacts need emails
          - You can exit the session at any time with the 'd' option
          - Empty email addresses are not allowed
          
        EXAMPLES:
          contact_fix_missing_emails
        """
        try:
            # Import the function here to avoid circular imports
            from contact.missing_email import interactive_email_manager
            
            # Call the interactive function
            interactive_email_manager()
            
        except ImportError as e:
            print(f"Error importing missing_email module: {e}", file=sys.stderr)
        except Exception as e:
            print(f"Error during interactive email management: {e}", file=sys.stderr)
            # Uncomment for debugging: traceback.print_exc(file=sys.stderr)
    
    def help_contact_fix_missing_emails(self):
        """Prints help for contact_fix_missing_emails."""
        print(self.do_contact_fix_missing_emails.__doc__)


# Main Application Entry Point
def handle_natural_language_fallback(command_string: str):
    """
    Handle natural language commands when regular command parsing fails.
    
    Args:
        command_string: The full command string entered by the user
    """
    from llm import init_llm_on_first_use, ensure_llm_model
    from nl_handler import NLCommandHandler
    from llm.manager import get_config
    
    # Check if LLM is available and enabled
    config = get_config()
    
    # If enable_llm is not set, prompt for first-time setup
    if "enable_llm" not in config:
        model_path = init_llm_on_first_use()
    else:
        model_path = ensure_llm_model()
    
    if not model_path:
        # LLM is disabled or not available
        print(f"Error: Unknown command. Natural language processing is disabled.")
        print(f"\nTo enable natural language commands, run:")
        print(f"  contactcmd llm enable")
        print(f"\nFor help, run: contactcmd --help")
        sys.exit(1)
    
    # Process the command with natural language
    try:
        print(f"\n🤖 Processing natural language command...")
        handler = NLCommandHandler(model_path)
        result = handler.process_command(command_string)
        
        if result:
            print(f"\n💡 Suggested command: {result}")
            
            # Ask if user wants to execute the command
            try:
                response = input("\nExecute this command? [Y/n]: ").strip().lower()
                if response in ['', 'y', 'yes']:
                    # Re-run with the suggested command
                    import subprocess
                    # Get the current script path
                    script_path = sys.argv[0]
                    # Execute the suggested command
                    subprocess.run([sys.executable, script_path] + result.split())
                else:
                    print("Command not executed.")
            except (EOFError, KeyboardInterrupt):
                print("\nCommand not executed.")
        else:
            print("Could not understand the command. Please try rephrasing or use --help for available commands.")
            sys.exit(1)
    except Exception as e:
        print(f"Error processing natural language: {e}")
        traceback.print_exc()
        sys.exit(1)


def main():
    """
    Main entry point for the contactcmd-cli application.
    Handles switching between Typer CLI mode and interactive shell mode.
    """
    # Ensure config directory exists for database and history file
    config_dir = os.path.join(os.path.expanduser("~"), ".config", "contactcmd")
    try:
        os.makedirs(config_dir, exist_ok=True)
    except OSError as e:
        print(f"Error creating config directory {config_dir}: {e}", file=sys.stderr)
        sys.exit(1) # Cannot proceed without config directory

    # Check if command-line arguments were provided (excluding the script name itself)
    if len(sys.argv) > 1:
        # Arguments are present, run as a standard CLI command via Typer
        try:
            # Call the main Typer application object
            app() # Typer automatically parses sys.argv[1:]
        except SystemExit as e:
            # Typer/Click uses SystemExit for success/failure (e.g., help message, validation error)
            # Check if this is a command not found error (exit code 2)
            if e.code == 2:
                # Try to handle as natural language
                command_string = ' '.join(sys.argv[1:])
                handle_natural_language_fallback(command_string)
            else:
                # Re-raise for other cases (help, normal exit, etc.)
                raise
        except UsageError as e:
            # This is a Click/Typer usage error (unknown command)
            command_string = ' '.join(sys.argv[1:])
            handle_natural_language_fallback(command_string)
        except Exception as e:
            # Check if this is a "No such command" error from Typer
            error_str = str(e)
            if "No such command" in error_str or "Invalid value" in error_str:
                command_string = ' '.join(sys.argv[1:])
                handle_natural_language_fallback(command_string)
            else:
                print(f"An unexpected error occurred during single command execution: {e}", file=sys.stderr)
                traceback.print_exc() # Print traceback for unexpected errors
                sys.exit(1) # Exit with an error code
    else:
        # No arguments, enter interactive mode
        try:
            ContactCMDShell().cmdloop() # Start the interactive loop
        except KeyboardInterrupt:
            print("\nExiting contactcmd shell.") # Handle Ctrl+C during the prompt
            sys.exit(0) # Exit cleanly
        except Exception as e:
            print(f"An unexpected error occurred during interactive mode: {e}", file=sys.stderr)
            traceback.print_exc() # Print traceback
            sys.exit(1) # Exit with an error code

# Run the application
if __name__ == "__main__":
    main() # Call the new main function