"""
Advanced database caching system with optimized memory usage, granular TTL settings,
and adaptive caching strategies for optimal performance.
"""
import threading
import time
import functools
import inspect
import hashlib
import logging
import weakref
from typing import Dict, Any, Tuple, Callable, Optional, Union, List, Set
from enum import Enum, auto
from dataclasses import dataclass
import os
import platform
import gc

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("cache")

# Cache statistics for performance monitoring
class CacheStats:
    """
    Cache statistics collection to track cache performance and optimize settings
    """
    # Thread-safe lock for stats
    _lock = threading.RLock()
    
    # Hit/miss counts by cache namespace
    hits: Dict[str, int] = {}
    misses: Dict[str, int] = {}
    
    # Memory usage tracking
    estimated_size: Dict[str, int] = {}
    
    # Invalidation tracking
    invalidations: Dict[str, int] = {}
    
    # Time-based stats
    avg_hit_latency: Dict[str, float] = {}
    
    @classmethod
    def record_hit(cls, namespace: str, latency: float) -> None:
        """Record a cache hit with its latency"""
        with cls._lock:
            cls.hits[namespace] = cls.hits.get(namespace, 0) + 1
            # Update average latency with moving average
            count = cls.hits[namespace]
            current_avg = cls.avg_hit_latency.get(namespace, 0)
            cls.avg_hit_latency[namespace] = current_avg + (latency - current_avg) / count
    
    @classmethod
    def record_miss(cls, namespace: str) -> None:
        """Record a cache miss"""
        with cls._lock:
            cls.misses[namespace] = cls.misses.get(namespace, 0) + 1
    
    @classmethod
    def record_invalidation(cls, namespace: str) -> None:
        """Record a cache invalidation"""
        with cls._lock:
            cls.invalidations[namespace] = cls.invalidations.get(namespace, 0) + 1
    
    @classmethod
    def update_memory_usage(cls, namespace: str, size: int) -> None:
        """Update estimated memory usage"""
        with cls._lock:
            cls.estimated_size[namespace] = size
    
    @classmethod
    def get_hit_ratio(cls, namespace: str) -> float:
        """Get hit ratio for a namespace"""
        with cls._lock:
            hits = cls.hits.get(namespace, 0)
            misses = cls.misses.get(namespace, 0)
            total = hits + misses
            return hits / total if total > 0 else 0
    
    @classmethod
    def get_all_stats(cls) -> Dict[str, Dict[str, Union[int, float]]]:
        """Get all stats grouped by namespace"""
        with cls._lock:
            # Combine all namespaces
            all_namespaces = set(cls.hits.keys()).union(cls.misses.keys())
            
            # Calculate stats for each namespace
            stats = {}
            for namespace in all_namespaces:
                hits = cls.hits.get(namespace, 0)
                misses = cls.misses.get(namespace, 0)
                total = hits + misses
                
                stats[namespace] = {
                    'hits': hits,
                    'misses': misses,
                    'total': total,
                    'hit_ratio': hits / total if total > 0 else 0,
                    'avg_hit_latency': cls.avg_hit_latency.get(namespace, 0),
                    'invalidations': cls.invalidations.get(namespace, 0),
                    'estimated_size': cls.estimated_size.get(namespace, 0)
                }
            
            return stats

class CacheEvictionPolicy(Enum):
    """Enum for cache eviction policies"""
    LRU = auto()  # Least Recently Used
    LFU = auto()  # Least Frequently Used
    FIFO = auto() # First In First Out

# Dataclass to store cache item metadata for advanced eviction policies
@dataclass
class CacheItemMetadata:
    """Metadata for cache items to support advanced eviction policies"""
    # When the item was last accessed (for LRU)
    last_accessed: float = 0.0
    # How many times the item was accessed (for LFU)
    access_count: int = 0
    # When the item was created (for FIFO)
    created_at: float = 0.0
    # Estimated size in bytes
    size_bytes: int = 0

# Default TTL settings optimized for different data types
DEFAULT_TTL_SETTINGS = {
    # Lookup or reference data (rarely changes)
    'reference': 3600,  # 1 hour
    # Standard data
    'standard': 300,    # 5 minutes
    # Frequently changing data
    'volatile': 60,     # 1 minute
    # Search results (common repeated searches)
    'search': 120,      # 2 minutes
    # User-specific data
    'user': 600,        # 10 minutes
    # System settings
    'settings': 1800,   # 30 minutes
}

# Context-aware maximum cache size based on platform and available memory
MAX_CACHE_SIZE = 50 * 1024 * 1024  # 50 MB default
if platform.system() == 'Darwin':  # macOS
    # macOS might have more memory available for desktop apps
    MAX_CACHE_SIZE = 100 * 1024 * 1024  # 100 MB
elif platform.system() == 'Windows':
    # Windows has good memory management for desktop apps
    MAX_CACHE_SIZE = 100 * 1024 * 1024  # 100 MB
else:  # Linux and others
    try:
        # Try to get memory info from /proc/meminfo
        with open('/proc/meminfo', 'r') as f:
            mem_info = {}
            for line in f:
                key, value = line.split(':')
                mem_info[key.strip()] = int(value.strip().split()[0])  # in kB
        
        # Set cache size to 2% of available memory
        if 'MemAvailable' in mem_info:
            MAX_CACHE_SIZE = mem_info['MemAvailable'] * 1024 * 0.02
    except:
        # Fallback to default if we can't read memory info
        pass

class AdvancedDbCache:
    """
    Advanced thread-safe database cache with multiple eviction policies,
    memory limits, and optimized key handling
    """
    
    # Class-level cache dictionaries and locks for thread safety
    _cache: Dict[str, Tuple[Any, float]] = {}
    _metadata: Dict[str, CacheItemMetadata] = {}
    _namespaces: Dict[str, Set[str]] = {}
    _lock = threading.RLock()
    
    # Cache configuration
    _eviction_policy = CacheEvictionPolicy.LRU
    _max_cache_size = MAX_CACHE_SIZE
    _current_size = 0
    
    @classmethod
    def _generate_key(cls, namespace: str, key_parts: List[Any]) -> str:
        """
        Generate a consistent, compact cache key using hashing for long keys
        
        Args:
            namespace: Cache namespace for organization
            key_parts: Parts to combine into a key
            
        Returns:
            A consistent string key
        """
        # Convert key parts to a string representation
        key_str = ":".join(str(p) for p in key_parts)
        
        # For very long keys, use a hash to keep cache keys small
        if len(key_str) > 200:
            # Use a combination of prefix and hash for debugging while keeping keys small
            prefix = key_str[:50]
            key_hash = hashlib.md5(key_str.encode()).hexdigest()
            final_key = f"{namespace}:{prefix}..{key_hash}"
        else:
            final_key = f"{namespace}:{key_str}"
            
        return final_key
    
    @classmethod
    def _estimate_size(cls, value: Any) -> int:
        """
        Estimate the memory size of a value in bytes
        
        Args:
            value: The value to estimate the size of
            
        Returns:
            Estimated size in bytes
        """
        # Start with a base size
        size = 64  # Base overhead
        
        # Check if the object has a __sizeof__ method for more accurate size
        if hasattr(value, '__sizeof__'):
            try:
                size += value.__sizeof__()
            except:
                pass
        
        # For special container types, include their content size
        if isinstance(value, (list, tuple, set)):
            # Add size for each item (estimation)
            for item in value:
                # Recursively estimate size, but with a limit to avoid infinite recursion
                # Just add a fixed size for simplicity
                size += 100  # Simplified estimation for contained objects
        
        elif isinstance(value, dict):
            # Add size for each key/value pair (estimation)
            for k, v in value.items():
                # Simplified estimation for keys and values
                size += 200  # Combined key+value approximation
        
        # For large complex objects, check if object has a size attribute
        elif hasattr(value, 'size') and callable(getattr(value, 'size', None)):
            try:
                size += value.size()
            except:
                pass
        
        return size
    
    @classmethod
    def _update_metadata(cls, key: str, value: Any) -> None:
        """Update metadata for a cache item for eviction policies"""
        now = time.time()
        
        # Create or update metadata
        if key in cls._metadata:
            metadata = cls._metadata[key]
            metadata.last_accessed = now
            metadata.access_count += 1
        else:
            size = cls._estimate_size(value)
            cls._metadata[key] = CacheItemMetadata(
                last_accessed=now,
                access_count=1,
                created_at=now,
                size_bytes=size
            )
            
            # Update current cache size
            cls._current_size += size
            
            # Update namespace stats
            namespace = key.split(':', 1)[0]
            CacheStats.update_memory_usage(namespace, cls._current_size)
            
    @classmethod
    def _evict_if_needed(cls) -> None:
        """Evict items if cache is too large based on eviction policy"""
        # Check if we need to evict
        if cls._current_size <= cls._max_cache_size:
            return
            
        # Sort keys based on eviction policy
        if cls._eviction_policy == CacheEvictionPolicy.LRU:
            # Sort by last accessed time (oldest first)
            sorted_keys = sorted(
                cls._metadata.keys(),
                key=lambda k: cls._metadata[k].last_accessed
            )
        elif cls._eviction_policy == CacheEvictionPolicy.LFU:
            # Sort by access count (least frequent first)
            sorted_keys = sorted(
                cls._metadata.keys(),
                key=lambda k: cls._metadata[k].access_count
            )
        else:  # FIFO
            # Sort by creation time (oldest first)
            sorted_keys = sorted(
                cls._metadata.keys(),
                key=lambda k: cls._metadata[k].created_at
            )
        
        # Evict items until we're under the size limit or out of items
        for key in sorted_keys:
            if cls._current_size <= cls._max_cache_size * 0.8:  # Add some margin
                break
                
            if key in cls._cache and key in cls._metadata:
                # Update size tracking
                cls._current_size -= cls._metadata[key].size_bytes
                
                # Update namespace tracking
                namespace = key.split(':', 1)[0]
                if namespace in cls._namespaces and key in cls._namespaces[namespace]:
                    cls._namespaces[namespace].remove(key)
                
                # Remove from cache and metadata
                del cls._cache[key]
                del cls._metadata[key]
                
                # Record eviction as an invalidation
                CacheStats.record_invalidation(namespace)
    
    @classmethod
    def get(cls, namespace: str, key_parts: List[Any]) -> Optional[Any]:
        """
        Get a cached value if it exists and hasn't expired
        
        Args:
            namespace: Cache namespace for organization
            key_parts: Parts to generate the key from
            
        Returns:
            The cached value if it exists and is still valid, otherwise None
        """
        key = cls._generate_key(namespace, key_parts)
        
        start_time = time.time()
        with cls._lock:
            if key in cls._cache:
                value, expires_at = cls._cache[key]
                if time.time() < expires_at:
                    # Update metadata for eviction policy
                    cls._update_metadata(key, value)
                    
                    # Record cache hit
                    CacheStats.record_hit(namespace, time.time() - start_time)
                    return value
                
                # Expired, remove from cache
                del cls._cache[key]
                
                # Update metadata and size tracking
                if key in cls._metadata:
                    cls._current_size -= cls._metadata[key].size_bytes
                    del cls._metadata[key]
                
                # Update namespace tracking
                if namespace in cls._namespaces and key in cls._namespaces[namespace]:
                    cls._namespaces[namespace].remove(key)
        
        # Record cache miss
        CacheStats.record_miss(namespace)
        return None
    
    @classmethod
    def set(cls, namespace: str, key_parts: List[Any], value: Any, ttl: Optional[int] = None) -> None:
        """
        Cache a value with a TTL in seconds
        
        Args:
            namespace: Cache namespace for organization
            key_parts: Parts to generate the key from
            value: The value to cache
            ttl: Time to live in seconds (default: based on namespace)
        """
        key = cls._generate_key(namespace, key_parts)
        
        # Use default TTL for the namespace if not specified
        if ttl is None:
            ttl = DEFAULT_TTL_SETTINGS.get(namespace, 300)  # 5 min default
        
        with cls._lock:
            # Store in cache with expiration time
            cls._cache[key] = (value, time.time() + ttl)
            
            # Update metadata for eviction policy
            cls._update_metadata(key, value)
            
            # Update namespace tracking
            if namespace not in cls._namespaces:
                cls._namespaces[namespace] = set()
            cls._namespaces[namespace].add(key)
            
            # Evict items if needed
            cls._evict_if_needed()
    
    @classmethod
    def delete(cls, namespace: str, key_parts: List[Any]) -> None:
        """
        Delete a specific key from the cache
        
        Args:
            namespace: Cache namespace for organization
            key_parts: Parts to generate the key from
        """
        key = cls._generate_key(namespace, key_parts)
        
        with cls._lock:
            if key in cls._cache:
                # Update size tracking
                if key in cls._metadata:
                    cls._current_size -= cls._metadata[key].size_bytes
                    del cls._metadata[key]
                
                # Remove from cache
                del cls._cache[key]
                
                # Update namespace tracking
                if namespace in cls._namespaces and key in cls._namespaces[namespace]:
                    cls._namespaces[namespace].remove(key)
                
                # Record invalidation
                CacheStats.record_invalidation(namespace)
    
    @classmethod
    def clear_namespace(cls, namespace: str) -> None:
        """
        Clear all items in a namespace
        
        Args:
            namespace: Cache namespace to clear
        """
        with cls._lock:
            if namespace in cls._namespaces:
                # Get all keys in this namespace
                keys_to_remove = cls._namespaces[namespace].copy()
                
                # Remove each key
                for key in keys_to_remove:
                    if key in cls._cache:
                        # Update size tracking
                        if key in cls._metadata:
                            cls._current_size -= cls._metadata[key].size_bytes
                            del cls._metadata[key]
                        
                        # Remove from cache
                        del cls._cache[key]
                
                # Clear namespace tracking
                cls._namespaces[namespace].clear()
                
                # Record invalidation
                CacheStats.record_invalidation(namespace)
    
    @classmethod
    def clear(cls) -> None:
        """Clear all cached items"""
        with cls._lock:
            # Clear all data
            cls._cache.clear()
            cls._metadata.clear()
            cls._namespaces.clear()
            cls._current_size = 0
            
            # Suggest garbage collection
            gc.collect()
    
    @classmethod
    def get_stats(cls) -> Dict[str, Any]:
        """Get cache statistics"""
        with cls._lock:
            stats = {
                'size': cls._current_size,
                'max_size': cls._max_cache_size,
                'usage_percent': (cls._current_size / cls._max_cache_size * 100) if cls._max_cache_size > 0 else 0,
                'items': len(cls._cache),
                'namespaces': len(cls._namespaces),
                'eviction_policy': cls._eviction_policy.name,
                'namespace_stats': CacheStats.get_all_stats()
            }
            
            return stats
    
    @classmethod
    def set_eviction_policy(cls, policy: CacheEvictionPolicy) -> None:
        """
        Set the cache eviction policy
        
        Args:
            policy: The eviction policy to use
        """
        with cls._lock:
            cls._eviction_policy = policy
    
    @classmethod
    def set_max_size(cls, max_size: int) -> None:
        """
        Set the maximum cache size in bytes
        
        Args:
            max_size: Maximum size in bytes
        """
        with cls._lock:
            cls._max_cache_size = max_size
            # Evict if needed with new size
            cls._evict_if_needed()

# Backward compatibility with the old DbCache
class DbCache(AdvancedDbCache):
    """Backward-compatible interface to the new cache system"""
    
    @classmethod
    def get(cls, key: str) -> Optional[Any]:
        """Get a cached value by key"""
        # Split namespace and key
        parts = key.split(':', 1)
        if len(parts) == 2:
            namespace, key_content = parts
        else:
            # Default namespace for backward compatibility
            namespace = 'default'
            key_content = key
            
        return AdvancedDbCache.get(namespace, [key_content])
    
    @classmethod
    def set(cls, key: str, value: Any, ttl: int = 60) -> None:
        """Set a cached value with TTL"""
        # Split namespace and key
        parts = key.split(':', 1)
        if len(parts) == 2:
            namespace, key_content = parts
        else:
            # Default namespace for backward compatibility
            namespace = 'default'
            key_content = key
            
        return AdvancedDbCache.set(namespace, [key_content], value, ttl)
    
    @classmethod
    def delete(cls, key: str) -> None:
        """Delete a cached value by key"""
        # Split namespace and key
        parts = key.split(':', 1)
        if len(parts) == 2:
            namespace, key_content = parts
        else:
            # Default namespace for backward compatibility
            namespace = 'default'
            key_content = key
            
        return AdvancedDbCache.delete(namespace, [key_content])
    
    @classmethod
    def clear(cls) -> None:
        """Clear all cached values"""
        return AdvancedDbCache.clear()

def cached_query(ttl: Optional[int] = None, key_prefix: str = "", cache_type: str = "standard") -> Callable:
    """
    Advanced decorator for caching database query results with context-aware TTL
    
    Args:
        ttl: Time to live in seconds (default: based on cache_type)
        key_prefix: Prefix for the cache key (default: "")
        cache_type: Type of cache for default TTL ("reference", "standard", "volatile", "search", "user", "settings")
        
    Returns:
        Decorated function that uses caching
    """
    # Determine namespace from prefix or cache_type
    namespace = key_prefix if key_prefix else cache_type
    
    # Get default TTL for this cache type if not specified
    if ttl is None:
        ttl = DEFAULT_TTL_SETTINGS.get(cache_type, 300)  # 5 min default
    
    def decorator(func: Callable) -> Callable:
        @functools.wraps(func)
        def wrapper(*args, **kwargs) -> Any:
            # Generate cache key components
            key_parts = [func.__name__]
            
            # Add arguments to key
            for arg in args:
                key_parts.append(str(arg))
            
            # Add sorted keyword arguments to key
            if kwargs:
                # Sort kwargs by key for consistent ordering
                sorted_kwargs = sorted(kwargs.items())
                for k, v in sorted_kwargs:
                    key_parts.append(f"{k}={v}")
            
            # Try to get the result from cache
            cached_result = AdvancedDbCache.get(namespace, key_parts)
            if cached_result is not None:
                return cached_result
            
            # Not in cache, call the original function
            result = func(*args, **kwargs)
            
            # Cache the result
            AdvancedDbCache.set(namespace, key_parts, result, ttl)
            
            return result
        return wrapper
    return decorator

# Intelligence-based selective caching decorator
def smart_cached_query(min_hits: int = 3, max_entries: int = 100, ttl: int = 300):
    """
    Intelligence-based selective caching that only caches frequently used queries
    
    Args:
        min_hits: Minimum number of identical queries before caching (default: 3)
        max_entries: Maximum number of tracked queries per function (default: 100)
        ttl: TTL for cached results in seconds (default: 300)
        
    Returns:
        Decorated function with smart caching
    """
    # Track query frequency
    query_counts = {}
    query_locks = {}
    
    def decorator(func: Callable) -> Callable:
        # Use function name for tracking
        func_name = func.__name__
        query_counts[func_name] = {}
        query_locks[func_name] = threading.RLock()
        
        @functools.wraps(func)
        def wrapper(*args, **kwargs) -> Any:
            # Generate cache key components
            key_parts = []
            
            # Add arguments to key
            for arg in args:
                key_parts.append(str(arg))
            
            # Add sorted keyword arguments to key
            if kwargs:
                # Sort kwargs by key for consistent ordering
                sorted_kwargs = sorted(kwargs.items())
                for k, v in sorted_kwargs:
                    key_parts.append(f"{k}={v}")
            
            # Create a hashable key for frequency tracking
            key_tuple = tuple(key_parts)
            
            # Try to get from cache first if we know it's a frequent query
            with query_locks[func_name]:
                is_frequent = key_tuple in query_counts[func_name] and query_counts[func_name][key_tuple] >= min_hits
            
            if is_frequent:
                # This is a frequent query, try to get from cache
                cached_result = AdvancedDbCache.get(f"smart_{func_name}", key_parts)
                if cached_result is not None:
                    return cached_result
            
            # Execute the query
            result = func(*args, **kwargs)
            
            # Update query frequency counters
            with query_locks[func_name]:
                # Limit tracked queries to prevent memory leaks
                if len(query_counts[func_name]) >= max_entries:
                    # Remove least frequent queries
                    sorted_items = sorted(query_counts[func_name].items(), key=lambda x: x[1])
                    query_counts[func_name] = dict(sorted_items[max_entries//2:])
                
                # Update count for this query
                query_counts[func_name][key_tuple] = query_counts[func_name].get(key_tuple, 0) + 1
                
                # Cache if this has become a frequent query
                if query_counts[func_name][key_tuple] >= min_hits:
                    AdvancedDbCache.set(f"smart_{func_name}", key_parts, result, ttl)
            
            return result
        
        return wrapper
    
    return decorator

# Example usage:
# @cached_query(cache_type="reference")  # Long TTL for reference data
# def get_country_by_code(country_code):
#     with Session() as session:
#         return session.query(Country).filter(Country.code == country_code).first()
#
# @cached_query(cache_type="search", ttl=120)  # Custom TTL for search results
# def search_contacts_by_name(name_part):
#     with Session() as session:
#         return session.query(Contact).filter(Contact.name.ilike(f"%{name_part}%")).all()
#
# @smart_cached_query(min_hits=2)  # Only cache after seen twice
# def get_contact_by_id(contact_id):
#     with Session() as session:
#         return session.query(Contact).filter(Contact.id == contact_id).first()