from sqlalchemy import create_engine, event
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.pool import QueuePool
import sys
import os
import platform

# Add parent directory to path to allow absolute imports
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from models.core import Base

def get_db_path():
    """Get platform-specific database path"""
    system = platform.system()
    
    if system == "Darwin":  # macOS
        db_dir = os.path.expanduser("~/.config/contactcmd")
    elif system == "Windows":
        db_dir = os.path.join(os.environ.get("APPDATA", ""), "ContactCMD")
    else:  # Linux/Unix
        db_dir = os.path.expanduser("~/.config/contactcmd")
    
    # Create directory if it doesn't exist
    os.makedirs(db_dir, exist_ok=True)
    
    return os.path.join(db_dir, "contactcmd.db")

def optimize_sqlite_connection(conn, _):
    """Apply performance optimizations to SQLite connection"""
    conn.execute('PRAGMA journal_mode = WAL')  # Write-Ahead Logging for better concurrency
    conn.execute('PRAGMA synchronous = NORMAL')  # Less robust but faster
    conn.execute('PRAGMA cache_size = 10000')   # Increase cache size
    conn.execute('PRAGMA temp_store = MEMORY')  # Store temp tables in memory
    conn.execute('PRAGMA mmap_size = 30000000')  # Memory-mapped I/O

# Create SQLite engine pointing to platform-specific location with connection pooling
db_path = get_db_path()
engine = create_engine(
    f"sqlite:///{db_path}",
    poolclass=QueuePool,
    pool_size=10,        # Increased pool size for better concurrency
    max_overflow=10,
    connect_args={"check_same_thread": False},  # SQLite-specific optimization
    echo=False  # Set to True for debug/development
)

# Apply SQLite optimizations to each connection in the pool
event.listen(engine, 'connect', optimize_sqlite_connection)

# Set up session with connection pooling
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)
Session = scoped_session(SessionLocal)

def create_all_tables():
    """Create all tables defined in the models"""
    Base.metadata.create_all(engine)

if __name__ == "__main__":
    print(f"Creating all tables in {db_path}...")
    create_all_tables()
    print("Done.")