#!/usr/bin/env python3
import sqlite3
import os
import sys
import typer
from db.database import get_db_path, optimize_sqlite_connection, engine
from models.core import Base

def create_index_if_not_exists(cursor, index_name, table_name, columns, unique=False):
    """
    Create an index if it doesn't exist
    
    Args:
        cursor: SQLite cursor
        index_name: Name of the index
        table_name: Name of the table
        columns: Comma-separated list of columns to index
        unique: Whether the index should be unique
    """
    # Check if index exists
    cursor.execute(f"SELECT name FROM sqlite_master WHERE type='index' AND name='{index_name}'")
    if not cursor.fetchone():
        unique_str = "UNIQUE" if unique else ""
        try:
            cursor.execute(f"CREATE {unique_str} INDEX {index_name} ON {table_name} ({columns})")
            print(f"Created index {index_name} on {table_name}({columns})")
            return True
        except sqlite3.Error as e:
            print(f"Error creating index {index_name}: {e}")
            return False
    else:
        print(f"Index {index_name} already exists")
        return True

def migrate_database():
    """
    Run migrations on the database to ensure it's up to date.
    
    This function:
    1. Adds any missing columns to existing tables
    2. Creates indexes for performance
    3. Applies SQLite optimizations
    4. Ensures the schema is compatible with the current model definitions
    """
    print("Running database migrations...")
    
    # Get the database path from database.py to ensure consistency
    db_path = get_db_path()
    print(f"Using database at: {db_path}")
    
    # First, ensure tables exist by creating them through SQLAlchemy
    try:
        print("Ensuring all tables exist...")
        Base.metadata.create_all(engine)
        print("Tables created or already exist")
    except Exception as e:
        print(f"Error creating tables: {e}")
    
    # Now perform direct SQLite operations for migrations and optimizations
    conn = sqlite3.connect(db_path)
    
    # Apply SQLite optimizations
    optimize_sqlite_connection(conn, None)
    
    cursor = conn.cursor()
    
    # Check if sync_notes column exists in contacts table
    cursor.execute("PRAGMA table_info(contacts)")
    columns = [info[1] for info in cursor.fetchall()]
    
    # Add missing columns if necessary
    if "sync_notes" not in columns:
        print("Adding 'sync_notes' column to contacts table...")
        try:
            cursor.execute("ALTER TABLE contacts ADD COLUMN sync_notes TEXT")
            conn.commit()
            print("Successfully added sync_notes column!")
        except sqlite3.Error as e:
            print(f"Error adding sync_notes column: {e}")
            conn.rollback()
    
    if "sync_status" not in columns:
        print("Adding 'sync_status' column to contacts table...")
        try:
            cursor.execute("ALTER TABLE contacts ADD COLUMN sync_status TEXT DEFAULT 'PENDING_ADD'")
            conn.commit()
            print("Successfully added sync_status column!")
        except sqlite3.Error as e:
            print(f"Error adding sync_status column: {e}")
            conn.rollback()
    
    if "is_dirty" not in columns:
        print("Adding 'is_dirty' column to contacts table...")
        try:
            cursor.execute("ALTER TABLE contacts ADD COLUMN is_dirty BOOLEAN DEFAULT 0")
            conn.commit()
            print("Successfully added is_dirty column!")
        except sqlite3.Error as e:
            print(f"Error adding is_dirty column: {e}")
            conn.rollback()
    
    # Create indexes for the contacts table
    print("\nCreating indexes for faster queries...")
    
    # Contacts table indexes
    create_index_if_not_exists(cursor, "idx_contact_search", "contacts", "name_first, name_last, email")
    create_index_if_not_exists(cursor, "idx_contact_email", "contacts", "email")
    create_index_if_not_exists(cursor, "idx_contact_name", "contacts", "name_first, name_last")
    create_index_if_not_exists(cursor, "idx_contact_sync", "contacts", "sync_status, is_dirty")
    
    # Check if interactions table exists
    cursor.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='interactions'")
    if cursor.fetchone():
        # Interactions table indexes
        create_index_if_not_exists(cursor, "idx_interaction_contact", "interactions", "contact_id")
        create_index_if_not_exists(cursor, "idx_interaction_type_date", "interactions", "type, created_at")
        create_index_if_not_exists(cursor, "idx_interaction_sync", "interactions", "sync_status, is_dirty")
    
    # Check if special_dates table exists
    cursor.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='special_dates'")
    if cursor.fetchone():
        # Special dates table indexes
        create_index_if_not_exists(cursor, "idx_specialdate_contact", "special_dates", "contact_id")
        create_index_if_not_exists(cursor, "idx_specialdate_type_date", "special_dates", "type, date")
        create_index_if_not_exists(cursor, "idx_specialdate_sync", "special_dates", "sync_status, is_dirty")
    
    # Apply additional SQLite optimizations to the database file
    print("\nApplying SQLite performance optimizations...")
    try:
        # Enable WAL mode for better concurrency
        cursor.execute("PRAGMA journal_mode = WAL")
        # Set synchronous mode to NORMAL for better performance
        cursor.execute("PRAGMA synchronous = NORMAL")
        # Increase cache size
        cursor.execute("PRAGMA cache_size = 10000")
        # Store temp tables in memory
        cursor.execute("PRAGMA temp_store = MEMORY")
        # Enable memory-mapped I/O
        cursor.execute("PRAGMA mmap_size = 30000000")
        print("SQLite optimizations applied successfully")
    except sqlite3.Error as e:
        print(f"Error applying SQLite optimizations: {e}")
    
    # Run ANALYZE to update statistics used by the query planner
    print("\nUpdating SQLite statistics for query optimization...")
    try:
        cursor.execute("ANALYZE")
        print("Statistics updated successfully")
    except sqlite3.Error as e:
        print(f"Error updating statistics: {e}")
    
    # Close the connection
    conn.commit()
    conn.close()
    print("\nDatabase migration and optimization completed.")

if __name__ == "__main__":
    migrate_database()