#!/bin/bash
# ContactCMD Universal Installer
# This script installs ContactCMD with all dependencies

set -e

echo "======================================"
echo "   ContactCMD Installer v1.0"
echo "======================================"
echo

# Configuration
INSTALL_DIR="$HOME/.contactcmd"
BIN_DIR="/usr/local/bin"
PYTHON_VERSION="3.11"

# Colors for output
RED='\033[0;31m'
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
NC='\033[0m' # No Color

# Functions
print_error() {
    echo -e "${RED}❌ $1${NC}"
}

print_success() {
    echo -e "${GREEN}✅ $1${NC}"
}

print_info() {
    echo -e "${YELLOW}ℹ️  $1${NC}"
}

# Check system
check_system() {
    echo "Checking system requirements..."
    
    # Check OS
    if [[ "$OSTYPE" != "darwin"* ]] && [[ "$OSTYPE" != "linux-gnu"* ]]; then
        print_error "This installer supports macOS and Linux only"
        exit 1
    fi
    
    # Check for Python
    if command -v python3 &> /dev/null; then
        PYTHON_CMD="python3"
    elif command -v python &> /dev/null; then
        PYTHON_CMD="python"
    else
        print_error "Python 3 is required but not found"
        echo "Please install Python 3.8 or later"
        exit 1
    fi
    
    # Check Python version
    PYTHON_VERSION_CHECK=$($PYTHON_CMD -c 'import sys; print(sys.version_info >= (3, 8))')
    if [ "$PYTHON_VERSION_CHECK" != "True" ]; then
        print_error "Python 3.8 or later is required"
        exit 1
    fi
    
    print_success "System requirements met"
}

# Install function
install_contactcmd() {
    # Create installation directory
    print_info "Creating installation directory..."
    mkdir -p "$INSTALL_DIR"
    
    # Create virtual environment
    print_info "Setting up Python environment..."
    $PYTHON_CMD -m venv "$INSTALL_DIR/venv"
    
    # Activate virtual environment
    source "$INSTALL_DIR/venv/bin/activate"
    
    # Upgrade pip
    print_info "Upgrading pip..."
    pip install --upgrade pip setuptools wheel
    
    # Copy source files
    print_info "Copying ContactCMD files..."
    cp -r contact db models interaction special_date sync llm "$INSTALL_DIR/"
    cp contactcmd.py nl_handler.py config.py auth.py "$INSTALL_DIR/"
    
    # Install dependencies
    print_info "Installing dependencies..."
    pip install typer[all] sqlalchemy rich python-dotenv
    
    # Try to install optional dependencies
    print_info "Installing optional dependencies..."
    
    # LLM support
    if [[ "$OSTYPE" == "darwin"* ]]; then
        # macOS - try to install with Metal support
        pip install llama-cpp-python --config-settings cmake.args="-DLLAMA_METAL=on" ||         pip install llama-cpp-python ||         print_info "LLM support not installed (optional)"
    else
        # Linux
        pip install llama-cpp-python ||         print_info "LLM support not installed (optional)"
    fi
    
    # macOS Contacts support
    if [[ "$OSTYPE" == "darwin"* ]]; then
        pip install pyobjc-framework-Contacts pyobjc-framework-AddressBook ||         print_info "macOS Contacts sync not installed (optional)"
    fi
    
    # Create wrapper script
    print_info "Creating command-line wrapper..."
    cat > "$INSTALL_DIR/contactcmd_wrapper.sh" << 'WRAPPER_EOF'
#!/bin/bash
# ContactCMD wrapper script

# Get the directory where this script is located
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

# Activate virtual environment and run ContactCMD
source "$SCRIPT_DIR/venv/bin/activate"
python "$SCRIPT_DIR/contactcmd.py" "$@"
WRAPPER_EOF
    
    chmod +x "$INSTALL_DIR/contactcmd_wrapper.sh"
    
    # Create symlink
    print_info "Installing command-line tool..."
    if [ -w "$BIN_DIR" ]; then
        ln -sf "$INSTALL_DIR/contactcmd_wrapper.sh" "$BIN_DIR/contactcmd"
    else
        print_info "Installing to $BIN_DIR requires sudo"
        sudo ln -sf "$INSTALL_DIR/contactcmd_wrapper.sh" "$BIN_DIR/contactcmd"
    fi
    
    # Create first-run setup script
    cat > "$INSTALL_DIR/first_run_setup.py" << 'SETUP_EOF'
#!/usr/bin/env python3
"""First run setup for ContactCMD"""
import os
import sys
from pathlib import Path

# Add ContactCMD to path
sys.path.insert(0, str(Path(__file__).parent))

from llm.manager import get_config_path, download_model, save_config, get_config, MODEL_REGISTRY

def first_run_setup():
    config_path = get_config_path()
    config_file = config_path / "config.json"
    
    if not config_file.exists():
        print("=" * 60)
        print("🎉 Welcome to ContactCMD!")
        print("=" * 60)
        print()
        
        # Create config directory
        config_path.mkdir(parents=True, exist_ok=True)
        
        # Ask about LLM
        print("ContactCMD can use AI for natural language commands.")
        enable = input("Enable AI features? [Y/n]: ").strip().lower() != 'n'
        
        config = get_config()
        
        if enable:
            print("\nAvailable models:")
            for i, (model_id, info) in enumerate(MODEL_REGISTRY.items(), 1):
                print(f"{i}. {info['name']} ({info['size']})")
            
            choice = input("\nSelect model [1]: ").strip() or "1"
            try:
                model_id = list(MODEL_REGISTRY.keys())[int(choice) - 1]
            except:
                model_id = "tinyllama"
            
            print(f"\nDownloading {MODEL_REGISTRY[model_id]['name']}...")
            if download_model(model_id):
                config["enable_llm"] = True
                config["default_llm"] = MODEL_REGISTRY[model_id]["filename"]
            else:
                config["enable_llm"] = False
        else:
            config["enable_llm"] = False
        
        save_config(config)
        print("\n✅ Setup complete!")

if __name__ == "__main__":
    first_run_setup()
SETUP_EOF
    
    chmod +x "$INSTALL_DIR/first_run_setup.py"
    
    # Run first setup
    print_info "Running initial setup..."
    python "$INSTALL_DIR/first_run_setup.py"
    
    # Deactivate virtual environment
    deactivate
}

# Main installation flow
main() {
    echo "This installer will:"
    echo "  • Install ContactCMD to ~/.contactcmd"
    echo "  • Create command 'contactcmd' in /usr/local/bin"
    echo "  • Set up AI features (optional)"
    echo
    
    read -p "Continue? [Y/n] " -n 1 -r
    echo
    if [[ $REPLY =~ ^[Nn]$ ]]; then
        echo "Installation cancelled."
        exit 0
    fi
    
    # Check system
    check_system
    
    # Perform installation
    install_contactcmd
    
    # Success message
    echo
    print_success "Installation complete!"
    echo
    echo "You can now use ContactCMD by running: contactcmd"
    echo "Try: contactcmd --help"
    echo
    echo "To uninstall, run: rm -rf ~/.contactcmd && rm /usr/local/bin/contactcmd"
}

# Run main
main
