"""
LLM module initialization with auto-download capability
"""
import os
from pathlib import Path
from typing import Optional

def ensure_llm_model() -> Optional[str]:
    """
    Ensure at least one LLM model is available.
    Downloads the smallest model if none exist.
    Returns the path to an available model or None if LLM is disabled.
    """
    from .manager import get_config, list_installed_models, download_model, enable_llm, MODEL_REGISTRY
    
    config = get_config()
    
    # Check if LLM is explicitly disabled
    if not config.get("enable_llm", False):
        return None
    
    # Check if any models are installed
    installed = list_installed_models()
    if installed:
        # Return the default model if it exists
        default_model = config.get("default_llm", "")
        models_path = Path.home() / ".config" / "contactcmd" / "models"
        default_path = models_path / default_model
        if default_path.exists():
            return str(default_path)
        # Otherwise return the first installed model
        return str(installed[0][1])
    
    # No models installed - offer to download the smallest one
    print("\n🤖 Natural language features require a local LLM model.")
    print("No models found. Would you like to download TinyLlama (669 MB)?")
    print("This is a one-time download that enables natural language commands.")
    
    try:
        response = input("\nDownload TinyLlama? [Y/n]: ").strip().lower()
        if response in ['', 'y', 'yes']:
            print("\nDownloading TinyLlama...")
            if download_model("tinyllama"):
                # Enable LLM and set as default
                enable_llm(True)
                models_path = Path.home() / ".config" / "contactcmd" / "models"
                model_path = models_path / MODEL_REGISTRY["tinyllama"]["filename"]
                return str(model_path)
            else:
                print("Download failed. Natural language features will be disabled.")
                return None
        else:
            print("\nSkipping download. You can enable this later with:")
            print("  contactcmd llm download tinyllama")
            print("  contactcmd llm enable")
            return None
    except (EOFError, KeyboardInterrupt):
        print("\nSkipping LLM setup.")
        return None

def init_llm_on_first_use():
    """
    Initialize LLM when natural language commands are first used.
    This is called lazily to avoid startup overhead.
    """
    from .manager import get_config, save_config
    
    config = get_config()
    
    # Check if this is the first run (no explicit enable/disable set)
    if "enable_llm" not in config:
        # First run - ask user if they want LLM features
        print("\n🤖 ContactCMD can understand natural language commands like:")
        print('  - "add john doe with email john@example.com"')
        print('  - "show recent interactions"')
        print('  - "list contacts added this week"')
        print("\nThis requires downloading a local AI model (669 MB).")
        
        try:
            response = input("\nEnable natural language features? [Y/n]: ").strip().lower()
            if response in ['', 'y', 'yes']:
                config["enable_llm"] = True
                save_config(config)
                # This will trigger the download prompt
                return ensure_llm_model()
            else:
                config["enable_llm"] = False
                save_config(config)
                print("\nNatural language features disabled. You can enable them later with:")
                print("  contactcmd llm enable")
                return None
        except (EOFError, KeyboardInterrupt):
            print("\nSkipping LLM setup.")
            config["enable_llm"] = False
            save_config(config)
            return None
    
    # Otherwise just ensure model if enabled
    return ensure_llm_model()