"""
LLM Model Manager for ContactCMD
Handles downloading, updating, and managing local LLM models
"""
import os
import json
import urllib.request
import urllib.error
import hashlib
import sys
from pathlib import Path
from typing import Dict, List, Optional, Tuple

# Model registry with recommended models
MODEL_REGISTRY = {
    "tinyllama": {
        "name": "TinyLlama 1.1B Chat",
        "filename": "tinyllama-1.1b-chat-v1.0.Q4_0.gguf",
        "size": "669 MB",
        "url": "https://huggingface.co/TheBloke/TinyLlama-1.1B-Chat-v1.0-GGUF/resolve/main/tinyllama-1.1b-chat-v1.0.Q4_0.gguf",
        "sha256": "260d6515cf225e5b3be8ff93ca8e8b59d0c7f0a5e3ff2bec66cb822279b0a332",
        "description": "Smallest model, fast responses, good for basic commands"
    },
    "phi-2": {
        "name": "Phi-2 2.7B", 
        "filename": "phi-2.Q4_K_M.gguf",
        "size": "1.6 GB",
        "url": "https://huggingface.co/TheBloke/phi-2-GGUF/resolve/main/phi-2.Q4_K_M.gguf",
        "sha256": "b4dc67856624feba008b6de75f231c09e7cf3f3b942c844df5dbbe257e0cecc6",
        "description": "Balanced model, better understanding, moderate speed"
    },
    "mistral-7b": {
        "name": "Mistral 7B Instruct",
        "filename": "mistral-7b-instruct-v0.2.Q4_K_M.gguf", 
        "size": "4.1 GB",
        "url": "https://huggingface.co/TheBloke/Mistral-7B-Instruct-v0.2-GGUF/resolve/main/mistral-7b-instruct-v0.2.Q4_K_M.gguf",
        "sha256": "6b42cc3583a3d2203fa0e4033be6c45a7a74c0a2aa13293e7fa8b4e7d2e7ba8f",
        "description": "Large model, best understanding, slower responses"
    }
}

def get_config_path() -> Path:
    """Get the ContactCMD configuration directory path"""
    return Path.home() / ".config" / "contactcmd"

def get_models_path() -> Path:
    """Get the models directory path"""
    config_path = get_config_path()
    return config_path / "models"

def get_config() -> Dict:
    """Load ContactCMD configuration"""
    config_file = get_config_path() / "config.json"
    if config_file.exists():
        with open(config_file, 'r') as f:
            return json.load(f)
    return {
        "database_path": str(get_config_path() / "contactcmd.db"),
        "models_path": str(get_models_path()),
        "default_llm": "tinyllama-1.1b-chat-v1.0.Q4_0.gguf",
        "enable_llm": False
    }

def save_config(config: Dict):
    """Save ContactCMD configuration"""
    config_file = get_config_path() / "config.json"
    config_file.parent.mkdir(parents=True, exist_ok=True)
    with open(config_file, 'w') as f:
        json.dump(config, f, indent=4)

def download_file_with_progress(url: str, filepath: Path, expected_size: Optional[int] = None) -> bool:
    """Download a file with progress bar"""
    try:
        # Create parent directory if needed
        filepath.parent.mkdir(parents=True, exist_ok=True)
        
        # Open URL
        response = urllib.request.urlopen(url)
        total_size = int(response.headers.get('Content-Length', 0))
        
        if expected_size and total_size != expected_size:
            print(f"Warning: Expected size mismatch (expected: {expected_size}, actual: {total_size})")
        
        # Download with progress
        downloaded = 0
        block_size = 8192
        
        with open(filepath, 'wb') as f:
            while True:
                chunk = response.read(block_size)
                if not chunk:
                    break
                    
                downloaded += len(chunk)
                f.write(chunk)
                
                # Show progress
                if total_size > 0:
                    percent = (downloaded / total_size) * 100
                    bar_length = 40
                    filled = int(bar_length * downloaded / total_size)
                    bar = '█' * filled + '░' * (bar_length - filled)
                    sys.stdout.write(f'\r[{bar}] {percent:.1f}% ({downloaded / 1024 / 1024:.1f} MB)')
                    sys.stdout.flush()
        
        print()  # New line after progress bar
        return True
        
    except Exception as e:
        print(f"\nError downloading file: {e}")
        if filepath.exists():
            filepath.unlink()  # Remove partial download
        return False

def verify_file_checksum(filepath: Path, expected_sha256: str) -> bool:
    """Verify file SHA256 checksum"""
    sha256_hash = hashlib.sha256()
    with open(filepath, "rb") as f:
        for byte_block in iter(lambda: f.read(4096), b""):
            sha256_hash.update(byte_block)
    
    actual_sha256 = sha256_hash.hexdigest()
    return actual_sha256 == expected_sha256

def list_installed_models() -> List[Tuple[str, Path, float]]:
    """List all installed models with their sizes"""
    models_path = get_models_path()
    installed = []
    
    if models_path.exists():
        for model_file in models_path.glob("*.gguf"):
            size_mb = model_file.stat().st_size / 1024 / 1024
            # Find model in registry
            model_id = None
            for mid, info in MODEL_REGISTRY.items():
                if info["filename"] == model_file.name:
                    model_id = mid
                    break
            installed.append((model_id or model_file.stem, model_file, size_mb))
    
    return installed

def download_model(model_id: str) -> bool:
    """Download a specific model"""
    if model_id not in MODEL_REGISTRY:
        print(f"Error: Unknown model '{model_id}'")
        print(f"Available models: {', '.join(MODEL_REGISTRY.keys())}")
        return False
    
    model_info = MODEL_REGISTRY[model_id]
    models_path = get_models_path()
    filepath = models_path / model_info["filename"]
    
    # Check if already exists
    if filepath.exists():
        print(f"Model {model_info['name']} already exists at {filepath}")
        print("Verifying checksum...")
        if verify_file_checksum(filepath, model_info["sha256"]):
            print("✅ Model is valid and ready to use")
            return True
        else:
            print("❌ Checksum mismatch, re-downloading...")
            filepath.unlink()
    
    print(f"Downloading {model_info['name']} ({model_info['size']})...")
    print(f"From: {model_info['url']}")
    print(f"To: {filepath}")
    
    if download_file_with_progress(model_info["url"], filepath):
        print("Verifying download...")
        if verify_file_checksum(filepath, model_info["sha256"]):
            print("✅ Download complete and verified!")
            return True
        else:
            print("❌ Download corrupted, please try again")
            filepath.unlink()
            return False
    
    return False

def set_default_model(model_filename: str):
    """Set the default LLM model"""
    config = get_config()
    config["default_llm"] = model_filename
    save_config(config)
    print(f"✅ Default model set to: {model_filename}")

def enable_llm(enable: bool = True):
    """Enable or disable LLM features"""
    config = get_config()
    config["enable_llm"] = enable
    save_config(config)
    status = "enabled" if enable else "disabled"
    print(f"✅ LLM features {status}")

def check_llm_status() -> Dict:
    """Check LLM configuration and status"""
    config = get_config()
    models_path = get_models_path()
    installed_models = list_installed_models()
    
    status = {
        "enabled": config.get("enable_llm", False),
        "default_model": config.get("default_llm", ""),
        "models_path": str(models_path),
        "installed_models": installed_models,
        "default_model_exists": False
    }
    
    # Check if default model exists
    if status["default_model"]:
        default_path = models_path / status["default_model"]
        status["default_model_exists"] = default_path.exists()
    
    return status