import datetime
import uuid
from sqlalchemy import Column, String, Boolean, Text, DateTime, Index
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
import sqlalchemy as sa

Base = declarative_base()

# Create a custom UUID type for SQLite
class GUID(sa.types.TypeDecorator):
    """Platform-independent GUID type.
    Uses UUID's string representation for SQLite.
    """
    impl = sa.String
    cache_ok = True
    
    def process_bind_param(self, value, dialect):
        if value is None:
            return None
        elif dialect.name == 'sqlite':
            return str(value)
        else:
            return str(value)
            
    def process_result_value(self, value, dialect):
        if value is None:
            return value
        else:
            if not isinstance(value, uuid.UUID):
                value = uuid.UUID(value)
            return value

class Contact(Base):
    __tablename__ = 'contacts'
    
    # Define table arguments with indexes for frequently queried fields
    __table_args__ = (
        # Index for search operations (name and email)
        Index('idx_contact_search', 'name_first', 'name_last', 'email'),
        
        # Index for email lookups
        Index('idx_contact_email', 'email'),
        
        # Index for name lookups
        Index('idx_contact_name', 'name_first', 'name_last'),
        
        # Index for sync operations
        Index('idx_contact_sync', 'sync_status', 'is_dirty'),
        
        # Better constraints to help SQLite optimizer
        {'sqlite_autoincrement': True}
    )
    
    # Use __slots__ to reduce memory usage and speed up attribute access
    __slots__ = ('_local_id', '_name_first', '_name_last', '_email', '_sync_notes', 
                '_created_at', '_updated_at', '_is_dirty', '_sync_status', '_last_synced_at',
                '_interactions', '_special_dates')
    
    local_id = Column(GUID(), primary_key=True, default=uuid.uuid4)
    name_first = Column(String)
    name_last = Column(String)
    email = Column(String)  # Simplified for tonight
    
    # Additional metadata
    sync_notes = Column(Text, nullable=True)  # For storing external IDs and sync metadata
    
    # Sync fields
    created_at = Column(DateTime, default=datetime.datetime.utcnow)
    updated_at = Column(DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
    is_dirty = Column(Boolean, default=False)
    sync_status = Column(String, default='PENDING_ADD')
    last_synced_at = Column(DateTime, nullable=True)
    
    # Relationships
    interactions = relationship('Interaction', back_populates='contact')
    special_dates = relationship('SpecialDate', back_populates='contact')
    
    # Optimized constructor for faster instantiation
    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
    
    def __repr__(self):
        return f"<Contact {self.name_first} {self.name_last}>"
    
    @property
    def full_name(self):
        return f"{self.name_first} {self.name_last}"
        
    # Cached property for frequently accessed operations
    _full_name_cache = None

    @property
    def full_name_cached(self) -> str:
        """Return cached full name and refresh if the name has changed."""
        current = f"{self.name_first} {self.name_last}"
        if self._full_name_cache != current:
            self._full_name_cache = current
        return self._full_name_cache