import datetime
import uuid
from sqlalchemy import Column, String, Boolean, Text, DateTime, ForeignKey, Index
from sqlalchemy.orm import relationship

from .core import Base, GUID

class Interaction(Base):
    __tablename__ = 'interactions'
    
    # Define table arguments with indexes for frequently queried fields
    __table_args__ = (
        # Index for contact lookups (most common query pattern)
        Index('idx_interaction_contact', 'contact_id'),
        
        # Index for type and date searches
        Index('idx_interaction_type_date', 'type', 'created_at'),
        
        # Index for sync operations
        Index('idx_interaction_sync', 'sync_status', 'is_dirty'),
        
        # Better constraints to help SQLite optimizer
        {'sqlite_autoincrement': True}
    )
    
    # Use __slots__ to reduce memory usage and speed up attribute access
    __slots__ = ('_local_id', '_contact_id', '_type', '_text', 
                '_created_at', '_updated_at', '_is_dirty', '_sync_status', '_last_synced_at',
                '_contact')
    
    local_id = Column(GUID(), primary_key=True, default=uuid.uuid4)
    contact_id = Column(GUID(), ForeignKey('contacts.local_id'))
    type = Column(String)
    text = Column(Text)
    
    # Sync fields
    created_at = Column(DateTime, default=datetime.datetime.utcnow)
    updated_at = Column(DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
    is_dirty = Column(Boolean, default=False)
    sync_status = Column(String, default='PENDING_ADD')
    last_synced_at = Column(DateTime, nullable=True)
    
    # Relationship
    contact = relationship('Contact', back_populates='interactions')
    
    # Optimized constructor for faster instantiation
    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
    
    def __repr__(self):
        return f"<Interaction {self.type} for contact_id={self.contact_id} on {self.created_at}>"

class SpecialDate(Base):
    __tablename__ = 'special_dates'
    
    # Define table arguments with indexes for frequently queried fields
    __table_args__ = (
        # Index for contact lookups (most common query pattern)
        Index('idx_specialdate_contact', 'contact_id'),
        
        # Index for date searches (e.g., upcoming birthdays)
        Index('idx_specialdate_type_date', 'type', 'date'),
        
        # Index for sync operations
        Index('idx_specialdate_sync', 'sync_status', 'is_dirty'),
        
        # Better constraints to help SQLite optimizer
        {'sqlite_autoincrement': True}
    )
    
    # Use __slots__ to reduce memory usage and speed up attribute access
    __slots__ = ('_local_id', '_contact_id', '_type', '_description', '_date',
                '_created_at', '_updated_at', '_is_dirty', '_sync_status', '_last_synced_at',
                '_contact')
    
    local_id = Column(GUID(), primary_key=True, default=uuid.uuid4)
    contact_id = Column(GUID(), ForeignKey('contacts.local_id'))
    type = Column(String)
    description = Column(String)
    date = Column(DateTime)
    
    # Sync fields
    created_at = Column(DateTime, default=datetime.datetime.utcnow)
    updated_at = Column(DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
    is_dirty = Column(Boolean, default=False)
    sync_status = Column(String, default='PENDING_ADD')
    last_synced_at = Column(DateTime, nullable=True)
    
    # Relationship
    contact = relationship('Contact', back_populates='special_dates')
    
    # Optimized constructor for faster instantiation
    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
    
    def __repr__(self):
        return f"<SpecialDate {self.type} for contact_id={self.contact_id} on {self.date}>"
        
    # Helper methods for common operations
    def is_upcoming(self, days=30):
        """Check if this special date is coming up within the specified number of days"""
        if not self.date:
            return False
        
        today = datetime.datetime.now().date()
        event_date = self.date.date()
        
        # Handle yearly events by replacing the year with the current year
        this_year_event = event_date.replace(year=today.year)
        if this_year_event < today:
            # Event already passed this year, check next year
            this_year_event = this_year_event.replace(year=today.year + 1)
            
        delta = (this_year_event - today).days
        return 0 <= delta <= days