import re
import os
import sys

try:
    from llama_cpp import Llama
    LLAMA_CPP_AVAILABLE = True
except ImportError:
    LLAMA_CPP_AVAILABLE = False
    Llama = None

class NLCommandHandler:
    def __init__(self, model_path="models/tinyllama-1.1b-chat-v1.0.q4_0.gguf", llm=None):
        """Initialize the command handler.

        Parameters
        ----------
        model_path : str
            Path to the GGUF model used when creating a default LLM instance.
        llm : object or type, optional
            Pre-initialized LLM instance or a class to instantiate. When not
            provided, :class:`llama_cpp.Llama` is created automatically.
        """
        self.model_path = model_path
        self.available_commands = []

        if llm is None:
            self._initialize_llm()
        else:
            if isinstance(llm, type):
                self.llm = llm(
                    model_path=self.model_path,
                    n_ctx=512,
                    n_threads=4,
                    n_gpu_layers=1,
                )
            else:
                self.llm = llm
            self.llm_available = True
    
    def _initialize_llm(self):
        """Initialize the LLM if the model file exists"""
        if not LLAMA_CPP_AVAILABLE:
            print("Natural language features require llama-cpp-python to be installed.")
            self.llm_available = False
            return
            
        # Try to use the provided model path first
        if os.path.exists(self.model_path):
            model_to_load = self.model_path
        else:
            # Try to get a model through auto-download
            try:
                from llm import init_llm_on_first_use
                model_to_load = init_llm_on_first_use()
            except ImportError:
                # If llm module not available, check config
                from pathlib import Path
                config_path = Path.home() / ".config" / "contactcmd"
                models_path = config_path / "models"
                
                # Look for any .gguf file
                if models_path.exists():
                    gguf_files = list(models_path.glob("*.gguf"))
                    if gguf_files:
                        model_to_load = str(gguf_files[0])
                    else:
                        model_to_load = None
                else:
                    model_to_load = None
            
            if not model_to_load:
                print(f"Natural language features are disabled.")
                self.llm_available = False
                return
        
        try:
            self.llm = Llama(
                model_path=model_to_load,
                n_ctx=512,
                n_threads=4,
                n_gpu_layers=1  # Use Metal GPU acceleration on Apple Silicon
            )
            self.llm_available = True
            print("NL command processing initialized successfully.")
        except Exception as e:
            print(f"Error initializing LLM: {e}")
            self.llm_available = False
    
    def register_commands(self, commands):
        """Register available commands with their descriptions"""
        self.available_commands = commands
    
    def _build_system_prompt(self):
        """Build the system prompt based on registered commands"""
        prompt = "You are an assistant that helps convert natural language requests into specific command line commands. "
        prompt += "Here are the available commands in this application:\n\n"
        
        for cmd in self.available_commands:
            prompt += f"- {cmd['command']}: {cmd['description']}\n"
        
        prompt += "\nWhen the user asks for something in natural language, respond with:"
        prompt += "\n1. A helpful explanation of what commands could help them"
        prompt += "\n2. Suggest a specific command in [square brackets]"
        prompt += "\n3. Ask if they want to execute that command"
        prompt += "\nOnly suggest commands from the list above."
        
        return prompt
    
    def process_input(self, user_input):
        """Process natural language input and return suggested command"""
        if not self.llm_available:
            return "Natural language processing is not available (model not loaded)."
        
        if not self.available_commands:
            return "No commands have been registered for processing."
        
        # Create the messages for the LLM
        messages = [
            {"role": "system", "content": self._build_system_prompt()},
            {"role": "user", "content": user_input}
        ]
        
        # Get the response from the LLM
        try:
            response = self.llm.create_chat_completion(messages=messages)
            return response['choices'][0]['message']['content']
        except Exception as e:
            return f"Error processing natural language: {e}"
    
    def extract_command(self, response):
        """Extract command in square brackets from the response"""
        match = re.search(r'\[(.*?)\]', response)
        if match:
            return match.group(1)
        return None
