import uuid
import datetime
import typer
from db.database import Session
from models import Contact, SpecialDate

def add_special_date(
    contact_id: str,
    type: str = typer.Option(..., "--type", help="Type of special date (e.g., birthday, anniversary)"),
    date: str = typer.Option(..., "--date", help="Date in YYYY-MM-DD format"),
    description: str = typer.Option("", "--description", help="Optional description for the special date")
):
    """
    Add a new special date for a contact
    """
    # Get a database session
    session = Session()
    try:
        # Convert string to UUID
        contact_uuid = uuid.UUID(contact_id)
        
        # Parse the date
        try:
            parsed_date = datetime.datetime.strptime(date, "%Y-%m-%d")
        except ValueError:
            typer.echo("Invalid date format. Please use YYYY-MM-DD")
            return
        
        # Check if contact exists
        contact = session.query(Contact).filter(Contact.local_id == contact_uuid).first()
        if not contact:
            typer.echo(f"Contact with ID {contact_id} not found.")
            return
        
        # Create a new special date
        special_date = SpecialDate(
            contact_id=contact_uuid,
            type=type,
            date=parsed_date,
            description=description,
            is_dirty=True,
            sync_status="PENDING_ADD"
        )
        
        # Add the special date to the database
        session.add(special_date)
        session.commit()
        
        # Print success message
        typer.echo(f"Special date added successfully for contact: {contact.name_first} {contact.name_last}")
        typer.echo(f"Special date ID: {special_date.local_id}")
        return special_date
    except ValueError:
        typer.echo(f"Invalid UUID format: {contact_id}", err=True)
    except Exception as e:
        session.rollback()
        typer.echo(f"Error adding special date: {str(e)}", err=True)
        raise
    finally:
        session.close()