#!/usr/bin/env python3
import sys
import os
import uuid
import datetime
from typing import Optional
import typer
from pathlib import Path

# Check if we're on macOS before importing Contacts
import platform
if platform.system() == 'Darwin':
    import Contacts  # macOS Contacts framework via PyObjC
    import objc  # PyObjC bridge
    import Foundation  # macOS Foundation framework

from db.database import Session
from models.core import Contact
from models.events import Interaction, SpecialDate
from sync.utils import get_last_sync_date, update_last_sync_date

def get_mac_contacts(since_date: Optional[datetime.datetime] = None):
    """
    Retrieve contacts from the macOS Contacts app.
    
    Args:
        since_date: If provided, only fetch contacts modified after this date
    
    Returns:
        List of contacts from macOS Contacts app
    """
    if platform.system() != 'Darwin':
        typer.echo("Error: macOS contacts sync is only available on macOS")
        return []
    
    contacts_list = []
    
    # Get the default ContactsBook
    contacts_book = Contacts.CNContactStore.alloc().init()
    
    # Request authorization to access contacts
    # First check the current authorization status (class method)
    status = Contacts.CNContactStore.authorizationStatusForEntityType_(Contacts.CNEntityTypeContacts)
    
    # Provide clear, helpful information based on authorization status
    if status == Contacts.CNAuthorizationStatusAuthorized:
        # Already authorized - proceed
        typer.echo("Permission to access Contacts is granted.")
    elif status == Contacts.CNAuthorizationStatusDenied:
        # Permission explicitly denied - show how to fix
        typer.echo("\n----------------------------------------")
        typer.echo("ERROR: Permission to access Contacts was denied.")
        typer.echo("")
        typer.echo("To fix this issue:")
        typer.echo("1. Open System Settings")
        typer.echo("2. Go to Privacy & Security → Contacts")
        typer.echo("3. Find Terminal (or your Python app) in the list")
        typer.echo("4. Check the box to allow access")
        typer.echo("5. If you don't see Terminal, click '+' to add it")
        typer.echo("   (Terminal is usually in /Applications/Utilities/)")
        typer.echo("6. Run this command again after granting permission")
        typer.echo("----------------------------------------\n")
        return contacts_list
    elif status == Contacts.CNAuthorizationStatusRestricted:
        # System policy restriction - explain this case
        typer.echo("\n----------------------------------------")
        typer.echo("ERROR: Access to Contacts is restricted by system policy.")
        typer.echo("This is usually due to parental controls or MDM restrictions.")
        typer.echo("Contact your system administrator to request access.")
        typer.echo("----------------------------------------\n")
        return contacts_list
    else:
        # Not determined - need to request permission
        typer.echo("\n----------------------------------------")
        typer.echo("Permission to access Contacts is required.")
        typer.echo("A system dialog will appear requesting permission.")
        typer.echo("Please click 'Allow' to continue with the sync.")
        typer.echo("----------------------------------------\n")
        
        # Use the asynchronous request with a callback and wait for response
        import threading
        permission_event = threading.Event()
        permission_granted = [False]
        permission_error = [None]
        
        def completion_handler(granted, error):
            permission_granted[0] = granted
            permission_error[0] = error
            permission_event.set()
        
        # Request permission
        contacts_book.requestAccessForEntityType_completionHandler_(
            Contacts.CNEntityTypeContacts, completion_handler
        )
        
        # Wait for permission response with a timeout
        if not permission_event.wait(timeout=15):
            typer.echo("\n----------------------------------------")
            typer.echo("ERROR: Timed out waiting for contacts permission.")
            typer.echo("The permission dialog may not have appeared or was ignored.")
            typer.echo("Please try running the command again.")
            typer.echo("----------------------------------------\n")
            return contacts_list
            
        if not permission_granted[0]:
            typer.echo("\n----------------------------------------")
            typer.echo("ERROR: Access to Contacts was denied.")
            typer.echo("")
            typer.echo("You denied permission to access your Contacts.")
            typer.echo("To enable sync functionality:")
            typer.echo("1. Open System Settings")
            typer.echo("2. Go to Privacy & Security → Contacts")
            typer.echo("3. Find Terminal (or your Python app) in the list")
            typer.echo("4. Check the box to allow access")
            typer.echo("5. Run this command again after granting permission")
            typer.echo("----------------------------------------\n")
            
            if permission_error[0]:
                typer.echo(f"System error: {permission_error[0]}")
                
            return contacts_list
        
        typer.echo("Permission to access Contacts was granted. Continuing with sync...")
    
    # Define which contact properties to fetch
    keys_to_fetch = [
        Contacts.CNContactGivenNameKey,
        Contacts.CNContactFamilyNameKey,
        Contacts.CNContactEmailAddressesKey,
        Contacts.CNContactPhoneNumbersKey,
        Contacts.CNContactBirthdayKey,
        Contacts.CNContactIdentifierKey,
    ]
    
    # Safely check if modification date key exists before adding it
    # This key might not exist in all macOS versions
    try:
        if hasattr(Contacts, 'CNContactModificationDateKey'):
            keys_to_fetch.append(Contacts.CNContactModificationDateKey)
    except (AttributeError, Exception) as e:
        if debug:
            typer.echo(f"DEBUG: ModificationDateKey not available: {e}")
        # Continue without the modification date key
    
    # Create fetch predicate if we have a since_date
    predicate = None
    if since_date:
        try:
            # Check if the predicateForContactsModifiedAfterDate_ method exists
            if hasattr(Contacts.CNContact, 'predicateForContactsModifiedAfterDate_'):
                # Convert Python datetime to NSDate
                import Foundation
                ns_date = Foundation.NSDate.dateWithTimeIntervalSince1970_(datetime.datetime.timestamp(since_date))
                predicate = Contacts.CNContact.predicateForContactsModifiedAfterDate_(ns_date)
            else:
                typer.echo("Warning: Cannot filter contacts by modification date. Getting all contacts instead.")
                # We'll continue without a predicate, fetching all contacts
        except (AttributeError, Exception) as e:
            if debug:
                typer.echo(f"DEBUG: Error creating date predicate: {e}")
            typer.echo("Warning: Cannot filter contacts by modification date. Getting all contacts instead.")
    
    # Create a fetch request
    fetch_request = Contacts.CNContactFetchRequest.alloc().initWithKeysToFetch_(keys_to_fetch)
    if predicate:
        fetch_request.setPredicate_(predicate)
    
    # Collect contacts
    error = None  # PyObjC handles the error parameter internally
    
    def gather_contacts(contact, stop_ptr):
        contacts_list.append(contact)
    
    try:
        # This method returns a boolean indicating success
        result = contacts_book.enumerateContactsWithFetchRequest_error_usingBlock_(
            fetch_request, None, gather_contacts
        )
        
        if not result:
            typer.echo("Error fetching contacts")
    except Exception as e:
        typer.echo(f"Error fetching contacts: {str(e)}")
    
    return contacts_list

def create_or_update_contact(mac_contact):
    """Create or update a contact in the contactcmd database from a macOS contact."""
    session = Session()
    try:
        # Extract basic contact information
        first_name = mac_contact.givenName()
        last_name = mac_contact.familyName()
        apple_contact_id = mac_contact.identifier()
        
        # Extract email (if available)
        email = None
        if mac_contact.emailAddresses() and len(mac_contact.emailAddresses()) > 0:
            email = mac_contact.emailAddresses()[0].value()
        
        # Extract phone (if available)
        phone = None
        if mac_contact.phoneNumbers() and len(mac_contact.phoneNumbers()) > 0:
            phone = mac_contact.phoneNumbers()[0].value().stringValue()
        
        # Get modification date (safely handle the case where the method doesn't exist)
        mod_date = None
        try:
            if hasattr(mac_contact, 'modificationDate') and callable(getattr(mac_contact, 'modificationDate', None)):
                mod_date = mac_contact.modificationDate()
        except (AttributeError, Exception):
            # Continue without modification date
            pass
        
        # First try to find by Apple contact ID in the metadata (stored in a sync notes field)
        try:
            existing_contact = session.query(Contact).filter(
                Contact.sync_notes.like(f'%apple_id:{apple_contact_id}%')
            ).first()
        except Exception as e:
            # Handle the case where sync_notes column might not exist
            if "no such column" in str(e):
                typer.echo("Warning: sync_notes column not found. Please run database migrations.")
                existing_contact = None
            else:
                raise e
        
        # If not found, try by email and name
        if not existing_contact and email:
            existing_contact = session.query(Contact).filter_by(
                name_first=first_name, 
                name_last=last_name,
                email=email
            ).first()
        
        result = False
        if not existing_contact:
            # Create new contact
            contact = Contact(
                name_first=first_name,
                name_last=last_name,
                email=email,
                is_dirty=True,  # Mark for future sync to server
                sync_notes=f"apple_id:{apple_contact_id}"  # Store Apple contact ID for future syncs
            )
            session.add(contact)
            
            # Add birthday as special date if available
            if mac_contact.birthday():
                birthday = mac_contact.birthday()
                
                # The birthday object is already an NSDateComponents object
                # We don't need to call dateComponents() on it
                
                # Convert NSDateComponents to Python datetime
                try:
                    year = birthday.year() or 1900  # Default if year is not set
                    # SQLite has limitations with date ranges - cap the year to a reasonable value
                    if year > 9000 or year < 1:  # Well beyond any realistic birth year
                        year = 2000  # Use a reasonable default
                    
                    month = birthday.month() or 1
                    day = birthday.day() or 1
                    
                    birthday_date = datetime.datetime(year, month, day)
                    
                    special_date = SpecialDate(
                        contact=contact,
                        type="birthday",
                        description=f"{first_name}'s birthday",
                        date=birthday_date,
                        is_dirty=True
                    )
                    session.add(special_date)
                except Exception as e:
                    # Log the error but continue with the contact import
                    typer.echo(f"Warning: Could not process birthday for {first_name} {last_name}: {str(e)}")
            
            # Commit the changes
            session.commit()
            typer.echo(f"Created new contact: {first_name} {last_name}")
            result = "Created"
        else:
            # Update existing contact if needed
            updated = False
            
            if existing_contact.name_first != first_name:
                existing_contact.name_first = first_name
                updated = True
                
            if existing_contact.name_last != last_name:
                existing_contact.name_last = last_name
                updated = True
                
            if email and existing_contact.email != email:
                existing_contact.email = email
                updated = True
            
            # Store Apple contact ID if not already stored
            try:
                # First check if sync_notes attribute exists and is accessible
                if hasattr(existing_contact, 'sync_notes'):
                    if 'apple_id:' not in (existing_contact.sync_notes or ''):
                        if existing_contact.sync_notes:
                            existing_contact.sync_notes += f";apple_id:{apple_contact_id}"
                        else:
                            existing_contact.sync_notes = f"apple_id:{apple_contact_id}"
                        updated = True
            except Exception as e:
                # Handle the case where sync_notes column might cause issues
                typer.echo(f"Warning: Could not update sync_notes: {str(e)}")
                # Continue without updating sync_notes
            
            if updated:
                existing_contact.is_dirty = True
                existing_contact.sync_status = 'PENDING_UPDATE'
                # Commit the changes
                session.commit()
                typer.echo(f"Updated existing contact: {first_name} {last_name}")
                result = "Updated"
            else:
                typer.echo(f"No changes for contact: {first_name} {last_name}")
        
        return result
    
    except Exception as e:
        session.rollback()
        typer.echo(f"Error processing contact: {str(e)}")
        return False
    finally:
        session.close()

def test_command():
    """Test command that doesn't access macOS APIs to verify basic functionality."""
    typer.echo("Sync module is working! This is a test command that doesn't use macOS APIs.")
    return

def sync_mac_contacts(
    all_contacts: bool = False,
    dry_run: bool = False,
    debug: bool = False
):
    """
    Sync contacts from macOS Contacts app to contactcmd.
    
    This command imports contacts from your Mac address book.
    Run with --help to see available options.
    """
    if platform.system() != 'Darwin':
        typer.echo("Error: macOS contacts sync is only available on macOS")
        sys.exit(1)
    
    # Run database migrations to ensure the schema is up to date
    try:
        sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
        from db.migrate import migrate_database
        migrate_database()
    except ImportError:
        typer.echo("Warning: Could not run database migrations. Some features may not work correctly.")
    except Exception as e:
        typer.echo(f"Warning: Error during database migration: {str(e)}")
    
    typer.echo("Starting macOS Contacts sync...")
    
    if debug:
        typer.echo("DEBUG MODE ENABLED")
        typer.echo(f"Python version: {sys.version}")
        typer.echo(f"Platform: {platform.platform()}")
        
        # List available constants in the Contacts module
        typer.echo("\nAvailable Contact keys:")
        contact_keys = [attr for attr in dir(Contacts) if attr.startswith('CNContact') and attr.endswith('Key')]
        for key in sorted(contact_keys):
            typer.echo(f"  - {key}")
        typer.echo("")
    
    try:
        if debug:
            typer.echo("DEBUG: Checking for PyObjC installation")
            try:
                import objc
                typer.echo(f"DEBUG: objc version: {objc.__version__}")
            except ImportError:
                typer.echo("DEBUG: objc module not found. This is required for Contacts framework.")
                return
        
        # Get the last sync date unless we want all contacts
        if debug:
            typer.echo("DEBUG: Getting last sync date")
        last_sync = None if all_contacts else get_last_sync_date()
        
        if last_sync:
            typer.echo(f"Getting contacts modified since {last_sync}")
        else:
            typer.echo("Getting all contacts from macOS Contacts app")
        
        # Get contacts from macOS Contacts app
        if debug:
            typer.echo("DEBUG: Retrieving contacts from macOS Contacts app")
        try:
            mac_contacts = get_mac_contacts(last_sync)
            if debug:
                typer.echo("DEBUG: Successfully retrieved contacts")
                typer.echo(f"DEBUG: Retrieved {len(mac_contacts)} contacts")
        except Exception as e:
            typer.echo("\n----------------------------------------")
            typer.echo(f"ERROR: {type(e).__name__} - {str(e)}")
            typer.echo("")
            typer.echo("This is likely due to one of the following issues:")
            typer.echo("")
            typer.echo("1. Permission Issues")
            typer.echo("   - Terminal needs permission to access Contacts")
            typer.echo("   - Go to System Settings → Privacy & Security → Contacts")
            typer.echo("   - Ensure Terminal is in the list and permission is granted")
            typer.echo("   - If not in the list, click '+' to add Terminal")
            typer.echo("")
            typer.echo("2. PyObjC Framework Issues")
            typer.echo("   - Make sure you have PyObjC installed correctly")
            typer.echo("   - Run: pip3 install -U pyobjc-framework-Contacts pyobjc-framework-ContactsUI")
            typer.echo("")
            typer.echo("3. macOS Version Issues")
            typer.echo("   - This functionality requires macOS 10.11 or later")
            typer.echo("   - The Contacts API may have changed in your macOS version")
            typer.echo("   - Try using the --all flag: python3 -m contactcmd sync mac --all")
            typer.echo("")
            typer.echo("For more information, see docs/MAC_SYNC.md")
            typer.echo("----------------------------------------\n")
            
            if debug:
                typer.echo("\nDetailed traceback for debugging:")
                import traceback
                typer.echo(traceback.format_exc())
            
            sys.exit(1)
        
        typer.echo(f"Found {len(mac_contacts)} contacts in macOS Contacts app")
        
        if dry_run:
            typer.echo("DRY RUN - No changes will be made")
            
            # Count how many would be created/updated
            new_count = 0
            update_count = 0
            
            session = Session()
            for mac_contact in mac_contacts:
                # Extract basic information for display
                first_name = mac_contact.givenName()
                last_name = mac_contact.familyName()
                apple_id = mac_contact.identifier()
                
                # Check if contact exists by Apple ID
                existing = session.query(Contact).filter(
                    Contact.sync_notes.like(f'%apple_id:{apple_id}%')
                ).first()
                
                if existing:
                    update_count += 1
                    typer.echo(f"Would update: {first_name} {last_name}")
                else:
                    new_count += 1
                    typer.echo(f"Would create: {first_name} {last_name}")
            
            session.close()
            typer.echo(f"DRY RUN SUMMARY: Would create {new_count} contacts and update {update_count} contacts")
                
        else:
            # Real run - make changes
            total_processed = 0
            created = 0
            updated = 0
            
            for mac_contact in mac_contacts:
                result = create_or_update_contact(mac_contact)
                total_processed += 1
                
                if total_processed % 10 == 0:
                    typer.echo(f"Processed {total_processed} contacts so far...")
                
                if result:
                    if result == "Created":
                        created += 1
                    elif result == "Updated":
                        updated += 1
            
            # Update the last sync timestamp
            update_last_sync_date()
            
            typer.echo(f"Sync completed successfully! Created {created} contacts and updated {updated} contacts.")
            
    except Exception as e:
        typer.echo(f"Error during sync: {str(e)}")
        sys.exit(1)

if __name__ == "__main__":
    app()